/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"

#include "connectmediasearch.h"

#include "mediamodel.h"
#include "searchplugin.h"
#include <mediaplugin.h>

#include <QDir>
#include <QPluginLoader>
#include <QSettings>

ConnectMediaSearch::ConnectMediaSearch(QWidget* parent, Qt::WindowFlags flags)
 : SettingsWidget(parent, flags)
{
	medium = 0;
	searchPlugin = 0;
	numMediaAttrs = 0;
	numSearchAttrs = 0;
	numMediaModelAttrs = 0;
	numSearchPluginAttrs = 0;
	canceled = false;
	deleteMedium = false;
	saved = false;
	
	setName(tr("Connect Media Attributes"));
  setupUi(this);
  connectionsView->setColumnCount(2);
  
  connect(assignButton, SIGNAL(clicked()), this, SLOT(assign()));
  connect(removeButton, SIGNAL(clicked()), this, SLOT(remove()));
  connect(mediaPluginAttrsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
  connect(searchPluginAttrsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
  connect(connectionsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
}

ConnectMediaSearch::~ConnectMediaSearch()
{
	if(deleteMedium)
		delete medium;
}

void ConnectMediaSearch::setPlugins(Medium *medium, SearchPlugin* searchPlugin)
{
  if(!medium)
    return;
  
  if(!searchPlugin)
    return;
	
	// replace medium with more specific one if needed
	if(medium->group().isEmpty() && medium->type() > 0){
		QSettings settings;
		QString pluginDir = settings.value ( "pluginDir" ).toString();
		if ( pluginDir.isEmpty() ) {
			pluginDir = QString ( PLUGINDIR );
		}
		//load and test plugin
		QDir dir ( pluginDir );
		if ( dir.exists() ) {
			QStringList files = dir.entryList ( QDir::Files );
			if ( files.size() > 0) {
				foreach ( QString file, files ) {
					QPluginLoader loader ( dir.filePath ( file ) );
					QObject *plugin = loader.instance();
					if ( plugin ) {
						MediaPlugin *newPlugin = qobject_cast<MediaPlugin *> ( plugin );
						if ( newPlugin ) {
							if(newPlugin->supportedMaterialTypes().contains(medium->typeName())){
								MediaModel *model = newPlugin->newModel(medium->typeName());
								medium = model->newMedium();
								deleteMedium = true;
								delete model;
								delete plugin;
								break;
							}
							delete plugin;
						}
					}
				}
			}
		}
	}
	
	this->medium = medium;
	mediaGroup = medium->group();
	mediaName = medium->name();
	mediaMap = medium->supportedAttrs();
	QStringList translations = medium->supportedAttributesTrans();
	this->searchPlugin = searchPlugin;

	loadSettings();
	
	mediaPluginAttrsView->clear();
	searchPluginAttrsView->clear();
	connectionsView->clear();
	connectionsView->setRowCount(0);
	
  mediaPluginNameLabel->setText(mediaName);
	translations.sort();
  mediaPluginAttrsView->addItems(translations);
	numMediaModelAttrs = mediaMap.size();
  
  searchPluginNameLabel->setText(searchPlugin->name());
  searchGroup = searchPlugin->settingsGroup();
	searchMap = searchPlugin->supportedAttributes();
	QStringList	items = searchMap.values();
	items.sort();
  searchPluginAttrsView->addItems(items);
	numSearchPluginAttrs = items.size();
  
  QStringList header;
  header << mediaName << searchPlugin->name();
  connectionsView->setHorizontalHeaderLabels(header);
	
	if(mediaAttrs.size() == searchAttrs.size()){
		connectionsView->setSortingEnabled(false);
		for(int i = 0; i < mediaAttrs.size(); i++){
			QList<QListWidgetItem*> itemsMediaAttrs = mediaPluginAttrsView->findItems(medium->attrs(mediaAttrs[i]).translation,
																																								Qt::MatchExactly);
			QList<QListWidgetItem*> itemsSearchAttrs = searchPluginAttrsView->findItems(searchMap.value(searchAttrs[i]),
																																									Qt::MatchExactly);
			if((itemsMediaAttrs.size() > 0) && (itemsSearchAttrs.size() > 0)){
				delete itemsMediaAttrs[0];
				delete itemsSearchAttrs[0];
				int row = connectionsView->rowCount();
				connectionsView->insertRow(row);
				connectionsView->setItem(row, 0, new QTableWidgetItem(medium->attrs(mediaAttrs[i]).translation));
				connectionsView->setItem(row, 1, new QTableWidgetItem(searchMap.value(searchAttrs[i])));
			}
		}
		connectionsView->setSortingEnabled(true);
	}
}

bool ConnectMediaSearch::saveSettings()
{
  if(mediaGroup.isEmpty())
    return false;
  if(searchGroup.isEmpty())
    return false;
  
  QStringList localMediaAttrs, localSearchAttrs;
  for(int i = 0; i < connectionsView->rowCount(); i++){
    QTableWidgetItem *item = 0;
    item = connectionsView->item(i, 0);
    if(!item)
      return false;
		foreach(ColKeyTrans keyItem, mediaMap){
			if(keyItem.translation == item->text())
				localMediaAttrs << keyItem.key;
		}
    
    item = connectionsView->item(i, 1);
    if(!item)
      return false;
    localSearchAttrs << searchMap.key(item->text());
  }
  
  QSettings settings;
  settings.beginGroup("mapping" + searchGroup + mediaGroup);
	settings.setValue("numMediaAttrs", numMediaModelAttrs);
	settings.setValue("numSearchAttrs", numSearchPluginAttrs);
  settings.setValue("mediaAttrs", localMediaAttrs);
  settings.setValue("searchAttrs", localSearchAttrs);
  settings.endGroup();
  
  return true;
}

void ConnectMediaSearch::assign()
{
  if(!mediaIndex.isValid())
    return;
  
  if(!searchIndex.isValid())
    return;
  
  QListWidgetItem *mediaItem = mediaPluginAttrsView->item(mediaIndex.row());
  if(!mediaItem)
    return;
  QString mediaAttr = mediaItem->text();
  
  QListWidgetItem *searchItem = searchPluginAttrsView->item(searchIndex.row());
  if(!searchItem)
    return;
  QString searchAttr = searchItem->text();
  
  connectionsView->setSortingEnabled(false);
  int row = connectionsView->rowCount();
  connectionsView->insertRow(row);
  connectionsView->setItem(row, 0, new QTableWidgetItem(mediaAttr));
  connectionsView->setItem(row, 1, new QTableWidgetItem(searchAttr));
  connectionsView->setSortingEnabled(true);
	
	delete searchItem;
	delete mediaItem;
}

void ConnectMediaSearch::remove()
{
  if(!connectionIndex.isValid())
    return;
  
  // add the remove attrs to the list of media model and search plugin
  QTableWidgetItem *item = 0;
  item = connectionsView->item(connectionIndex.row(), 0);
  if(!item)
    return;
  mediaPluginAttrsView->addItem(item->text());
  
  item = connectionsView->item(connectionIndex.row(), 1);
  if(!item)
    return;
  searchPluginAttrsView->addItem(item->text());
  
  connectionsView->removeRow(connectionIndex.row());
}

void ConnectMediaSearch::indexChanged(QModelIndex index)
{
  if(!sender())
    return;
  if(sender()->objectName() == "mediaPluginAttrsView"){
    mediaIndex = index;
    return;
  }
  if(sender()->objectName() == "searchPluginAttrsView"){
    searchIndex = index;
    return;
  }
  if(sender()->objectName() == "connectionsView"){
    connectionIndex = index;
    return;
  }
}

bool ConnectMediaSearch::checkSettings()
{
	if(!medium)
    return false;
  
  if(!searchPlugin)
    return false;
	
	loadSettings();
	// do some checks, if we must display the dialog
	bool result = true;
	// check wether the attribute number has changed
	if(numMediaAttrs != numMediaModelAttrs)
		result = false;
	if(numSearchAttrs != numSearchPluginAttrs)
		result = false;
	
	// check that all listed attributes are supported by the medium and plugin right now
	for(int i = 0; i < mediaAttrs.size(); i++){
		if(medium->supportedAttributes().indexOf(mediaAttrs[i]) == -1)
			result = false;
	}
	for(int i = 0; i < searchAttrs.size(); i++){
		if(searchPlugin->supportedAttributes().keys().indexOf(searchAttrs[i]) == -1)
			result = false;
	}

	if ( mediaAttrs.size() != searchAttrs.size() )
		result = false;
	
	return result;
}

void ConnectMediaSearch::loadSettings()
{
	if(!medium)
		return;
	
	if(!searchPlugin)
		return;
	
	// loading status quo
	QSettings settings;
	settings.beginGroup ( "mapping" + searchPlugin->settingsGroup() + mediaGroup );
	numMediaAttrs = settings.value("numMediaAttrs", 0).toInt();
	numSearchAttrs = settings.value("numSearchAttrs", 0).toInt();
	mediaAttrs = settings.value ( "mediaAttrs" ).toStringList();
	searchAttrs = settings.value ( "searchAttrs" ).toStringList();
	settings.endGroup();
}

#include "connectmediasearch.moc"
