/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmedium.h"

#include "medium.h"
#include "mediamodel.h"

#include <QDebug>

EditMedium::EditMedium(QWidget* parent, Qt::WindowFlags flags)
	: MDialog(parent, flags)
{
	medium = 0;
	model = 0;
	editMode = Edit;
	needSave = false;
	startTransaction = true;
	setupUi(this);
	setHelp("keyword:Edit Medium", helpButton);
	setWindowTitle(tr("Medium"));
	
	connect(okButton, SIGNAL(clicked()), this, SLOT(save()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
	
	connectByPass(titleWidget);
	connectByPass(manufacturerWidget);
	connectByPass(genreWidget);
	connectByPass(personsWidget);
	connectByPass(descriptionWidget);
	connectByPass(minWidget);
	connectByPass(wishWidget);
	connectByPass(literatureWidget);
	connectByPass(evaluationWidget);
	connectByPass(boughtPriceWidget);
	connectByPass(listPriceWidget);
	connectByPass(giftWidget);
	connectByPass(ownerWidget);
	connectByPass(loanWidget);
}

void EditMedium::setAddMode(Medium* medium, bool newTransaction)
{
	editMode = Add;
	setWindowTitle(tr("Add Medium to Database"));
	setMedium(medium, newTransaction);
}

void EditMedium::setEditMode(Medium* medium, bool newTransaction)
{
	editMode = Edit;
	setWindowTitle(tr("Edit Medium"));
	setMedium(medium, newTransaction);
}

void EditMedium::setNewMode(MediaModel *model, Medium *medium, bool newTransaction)
{
	editMode = New;
	if(!model)
		return;
	this->model = model;
	
	setWindowTitle(tr("Add Medium to Database"));
	setMedium(medium, newTransaction);
}

void EditMedium::save()
{
	if(!medium)
		return;
	
	if(!prepareSave(medium))
		return;
	
	if(!loanWidget->store()){
		emit error("Could not store loan changes");
		return;
	}
	
	if(!ownerWidget->store()){
		emit error("Could not store owner changes");
		return;
	}
	
	if(!giftWidget->store()){
		emit error("Could not store gift changes");
		return;
	}
	
	if(!descriptionWidget->store()){
		emit error("Could not store description changes");
		return;
	}
	
	switch(editMode){
		case New:
			if(!model)
				return;
			model->addMedium(medium);
			if(!medium->store(true, startTransaction))
				return;
			medium->setSource(Medium::Local);
			medium->setSourceIcon(QIcon ( ":icons/128x128/meganizer.png" ));
			break;
		case Add:
			if(!medium->store(true, startTransaction))
				return;
			medium->setSource(Medium::Local);
			medium->setSourceIcon(QIcon ( ":icons/128x128/meganizer.png" ));
			break;
		default:
			if(!medium->store(false, startTransaction))
				return;
			break;
	}
	medium->disconnect(this, SLOT(mediumChanged()));
	accept();
}

void EditMedium::insertWidget(QWidget* newWidget, QWidget* intoWidget, QWidget* afterWidget)
{
	insertItem(new QWidgetItem(newWidget), intoWidget, afterWidget);
}

void EditMedium::insertLayout(QLayout* newLayout, QWidget* intoWidget, QWidget* afterWidget)
{
	insertItem(newLayout, intoWidget, afterWidget);
}

QLabel* EditMedium::prepareLabel(QString text)
{
	QLabel *label = new QLabel(text, this);
	label->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	label->setMinimumSize(150, 30);
	label->setMaximumSize(150, 30);
	return label;
}

void EditMedium::cancel()
{
	// if nothing changed, we accept to avoid
	// missleading error messages
// 	if(!needSave)
		accept();
	
// 	reject();
}

void EditMedium::mediumChanged()
{
	needSave = true;
}

void EditMedium::setMedium(Medium* medium, bool newTransaction)
{
	needSave = true;
	if(medium)
		connect(medium, SIGNAL(changed(Medium*)), this, SLOT(mediumChanged()));
	startTransaction = newTransaction;
	
	titleWidget->setMedium(medium);
	manufacturerWidget->setMedium(medium);
	genreWidget->setMedium(medium);
	personsWidget->setMedium(medium);
	descriptionWidget->setMedium(medium);
	minWidget->setMedium(medium);
	wishWidget->setMedium(medium);
	literatureWidget->setMedium(medium);
	evaluationWidget->setMedium(medium);
	boughtPriceWidget->setMedium(medium);
	listPriceWidget->setMedium(medium);
	giftWidget->setMedium(medium);
	ownerWidget->setMedium(medium);
	loanWidget->setMedium(medium);
	imageWidget->setMedium(medium);
	keywordWidget->setMedium(medium);
	
	loadData(medium);
	
	this->medium = medium;
}

void EditMedium::insertItem(QLayoutItem* newItem, QWidget* intoWidget, QWidget* afterWidget)
{
	if(!intoWidget)
		return;
	if(!newItem)
		return;
	
	// determine target layout
	QLayout *targetLayout = intoWidget->layout();
	// handling special case meta data, because it's a QVBox in QHBox
	if(intoWidget->objectName() == "metaDataTab")
		targetLayout = metaDataLayout;
	if(!targetLayout)
		return;
		
	// determine index
	int index = targetLayout->count();
	if(!afterWidget){
		// insert it at last position BEFORE stretch
		if(targetLayout->count() > 0)
			if(targetLayout->itemAt(targetLayout->count() - 1)->spacerItem())
				index = targetLayout->count() - 1;
	}
	else{
		// if index is -1 (afterWidget is not found), the widget is appended
		index = targetLayout->indexOf(afterWidget) + 1;
	}

	QBoxLayout *la = qobject_cast<QBoxLayout*>(targetLayout);
	if(la){
		if(newItem->widget()){
			la->insertWidget(index, newItem->widget());
		}
		else if(newItem->layout()){
			la->insertLayout(index, newItem->layout());
		}
		else if(newItem->spacerItem()){
			la->insertSpacerItem(index, newItem->spacerItem());
		}
	}
// 	tabOrder(0, layout());
}

QWidget* EditMedium::tabOrder(QWidget* lastWidget, QLayout* childlayout)
{
	if(!childlayout)
		return lastWidget;
	
	for(int i = 0; i < childlayout->count(); i++){
		if(childlayout->itemAt(i)->widget()){
			QWidget *widget = childlayout->itemAt(i)->widget();
			QLabel *label = qobject_cast<QLabel*>(widget);
			if(!label){
				if(lastWidget){
					setTabOrder(lastWidget, widget);
					qDebug() << lastWidget->objectName() << " to " << widget->objectName();
				}
				lastWidget = widget;

				// handle special container widgets
				QTabWidget *tabWi = qobject_cast<QTabWidget*>(widget);
				if(tabWi){
					for(int j = 0; j < tabWi->count(); j++)
						lastWidget = tabOrder(lastWidget, tabWi->widget(j)->layout());
				}
			}
		}
		else if(childlayout->itemAt(i)->layout()){
			lastWidget = tabOrder(lastWidget, childlayout->itemAt(i)->layout());
		}
	}
	return lastWidget;
}

#include "editmedium.moc"
