/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "loanmodel.h"

#include "loanitem.h"
#include "delegateloanstati.h"
#include "delegateuser.h"
#include "msqldatabase.h"

#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QTableView>
#include <QAbstractItemView>

LoanModel::LoanModel ( unsigned int materialId, unsigned int ownUserId, QObject* parent )
	: QAbstractItemModel ( parent )
{
	init(materialId, ownUserId);
}

LoanModel::LoanModel ( unsigned int ownUserId, QObject* parent )
	: QAbstractItemModel ( parent )
{
	init(0, ownUserId, true);
}

LoanModel::~LoanModel()
{
	qDeleteAll(items);
}

int LoanModel::indexOf ( QString columnName ) const
{
	return headerList.indexOf(columnName);
}

LoanItem* LoanModel::loanItemFromIndex ( QModelIndex index ) const
{
	if(!index.isValid())
		return 0;
	return static_cast<LoanItem*>(index.internalPointer());
}

bool LoanModel::store()
{
	bool result = true;
	foreach(LoanItem *item, items){
		if(!item->store())
			result = false;
	}
	return result;
}

bool LoanModel::reload()
{
	return true;
}

bool LoanModel::clear()
{
	emit layoutAboutToBeChanged();
	items.clear();
	emit layoutChanged();
	if(items.size() == 0)
		return true;
	return false;
}

void LoanModel::addItemDelegates ( QAbstractItemView* view )
{
	if(!view)
		return;
	DelegateLoanStati *statusDelegate = new DelegateLoanStati(view);
	view->setItemDelegateForColumn(2, statusDelegate);
	DelegateUser *userDelegate = new DelegateUser(view);
	view->setItemDelegateForColumn(0, userDelegate);
	view->setItemDelegateForColumn(1, userDelegate);
}

void LoanModel::hideAndResizeColumns ( QTableView* view )
{
	if(!view)
		return;
	view->resizeColumnsToContents();
}

QVariant LoanModel::data ( const QModelIndex& index, int role ) const
{
	if(role != Qt::DisplayRole
			&& role != Qt::EditRole
			&& role != Qt::BackgroundRole
		)
		return QVariant();
	
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return QVariant();
	
	if(role == Qt::BackgroundRole){
		QDate dueDate = item->loanedUntil();
		if(dueDate.isValid() && dueDate < QDate::currentDate())
			return QBrush(QColor ( 244, 12, 13, 60 ));
		return QBrush();
	}
	
	switch(index.column()){
		case 0:
			return item->owner();
		case 1:
			return item->recipient();
		case 2:
			return item->status();
		case 3:
			return item->loanedUntil();
		case 4:
			return item->lastChange();
	}
	return QVariant();
}

int LoanModel::columnCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return headerList.size();
}

int LoanModel::rowCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return items.size();
}

QModelIndex LoanModel::parent ( const QModelIndex& child ) const
{
	return QModelIndex();
}

QModelIndex LoanModel::index ( int row, int column, const QModelIndex& parent ) const
{
	if(parent.isValid())
		return QModelIndex();
	
	if(column < 0)
		return QModelIndex();
	if(columnCount() <= column)
		return QModelIndex();
	
	if((row < 0) || (items.size() <= row))
		return QModelIndex();
	
	return createIndex(row, column, items[row]);
}

bool LoanModel::setData ( const QModelIndex& index, const QVariant& value, int role )
{
  if(role != Qt::EditRole)
		return false;
	
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return false;
	
	switch(index.column()){
		case 1:
			item->setRecipient(value.toUInt());
			return true;
		case 2:
			return item->setStatus(value.toUInt());
		case 3:
			return item->setLoanedUntil(value.toDate());
	}
	return false;
}

QVariant LoanModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
  if(orientation == Qt::Vertical)
		return section + 1;
	
	if((section < 0) || (headerList.size() <= section))
		return QVariant();
	
	return headerList[section];
}

Qt::ItemFlags LoanModel::flags ( const QModelIndex& index ) const
{
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return QAbstractItemModel::flags(index);

	switch(index.column()){
		case 1:
			return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		case 2:
			return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		case 3:
			if(item->owner() == myUserId || item->ownerUserName().isEmpty())
				return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
			else
				return QAbstractItemModel::flags(index);
	}
	return QAbstractItemModel::flags(index);
}

void LoanModel::sort ( int column, Qt::SortOrder order )
{
  QAbstractItemModel::sort ( column, order );
}

void LoanModel::errorByPass ( QString msg )
{
	emit error(msg);
}

void LoanModel::warningByPass ( QString msg )
{
	emit warning(msg);
}

void LoanModel::informationByPass ( QString msg )
{
	emit information(msg);
}

void LoanModel::itemChanged()
{
	emit dataChanged ( index ( 0, 0 ), index ( rowCount(), columnCount() ) );
}

void LoanModel::init(unsigned int materialId, unsigned int ownUserId, bool loadWithoutMaterial)
{
	this->materialId = materialId;
	myUserId = ownUserId;
	if(ownUserId == 0){
		MSqlDatabase db = MSqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.isOpen()){
			QSqlQuery query;
			query.prepare("SELECT id FROM users WHERE userName=:userName");
			query.bindValue(":userName", db.userName());
			query.exec();
			if(query.size() == 1)
				if(query.first())
					myUserId = query.record().value("id").toUInt();
			if(closeDB)
				db.close();
		}
	}

	setupHeader();
	if(materialId != 0 || loadWithoutMaterial)
		load();
}

void LoanModel::load()
{
	if((materialId == 0) && (myUserId == 0))
		return;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen())
		return;
	
	emit layoutAboutToBeChanged();
	
	QList<LoanItem*> finishedLoanes;
	QSqlQuery query;
	QString whereClause = QString("WHERE usermats.materialsId=%1").arg(materialId);
	if(materialId == 0)
		whereClause = QString("WHERE (usermats.usersId=%1 OR loaned.recipientId=%1)"
													"  AND loaned.id<>0"
													"  AND (loaned.status<>5 AND loaned.status<>2 AND loaned.status<>6)").arg(myUserId);
	
	query.exec(QString("SELECT usermats.materialsId, usermats.usersId, users.userName,"
													" loaned.id, loaned.recipientId, loaned.status, loaned.loanedUntil, loaned.lastChange"
													" FROM userownmaterials usermats"
													" LEFT JOIN loanedmaterials loaned ON usermats.materialsId=loaned.materialsId"
													" LEFT JOIN users ON usermats.usersId=users.id"
													" %1").arg(whereClause));
	while(query.next()){
		LoanItem *item = new LoanItem(this);
		item->setLoggedInUser(myUserId);
		connect(item, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
		connect(item, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
		connect(item, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
		connect(item, SIGNAL(changed()), this, SLOT(itemChanged()));
		
		if(!item->load(query.record())){
			delete item;
		}
		else{
			items << item;
			if(item->loanFinished())
				finishedLoanes.append(item);
		}
	}
	
	// strip out finished items
	QList<LoanItem*> canBeLoaned;
	foreach(LoanItem *finishedItem , finishedLoanes){
		// check wether this medium can be loaned right now
		foreach(LoanItem *item, items){
			if(item->material() != item->material())
				continue;
			if(item->owner() != finishedItem->owner())
				continue;
			if(!item->loanFinished())
				continue;

			// we're here, so it is the same medium of same owner and
			// it is not already loaned right now, so we can loan it again
			// But check, wether it is already marked for loaning
			bool alreadyPresent = false;
			foreach(LoanItem *canBeLoanedItem, canBeLoaned){
				if(canBeLoanedItem->material() == finishedItem->material()){
					if(canBeLoanedItem->owner() == finishedItem->owner()){
						alreadyPresent = true;
						break;
					}
				}
			}
			if(!alreadyPresent)
				canBeLoaned << finishedItem;
			break;
		}
	}

	// Before inserting loanable items, strip out all old loanes from this medium and owner
	// from model
	foreach(LoanItem *canBeItem, canBeLoaned){
		foreach(LoanItem *item, items){
			if(item != canBeItem){
				if(item->material() == canBeItem->material()
					&& item->owner() == canBeItem->owner()
					&& item->loanFinished()){
							items.removeAll(item);
							delete item;
				}
			}
			else{
				// set up item correctly (ignore rights to be sure changes are mode)
				canBeItem->setRecipient(0, true);
				canBeItem->setStatus(0, true);
				canBeItem->setLoanedUntil(QDate());
				canBeItem->setLastChange(QDate());
			}
		}
	}
	
	emit layoutChanged();
	
	if(closeDB)
		db.close();
}

void LoanModel::setupHeader()
{
	headerList << tr ( "Owner" ) << tr ( "Loaner" );
	headerList << tr ( "Status" ) << tr ( "Due Date" );
	headerList << tr ( "Last Change" );
}

#include "loanmodel.moc"
