/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIAMODEL_H
#define MEDIAMODEL_H

#include "libmacro.h"

#include <QAbstractItemModel>

#include "medium.h"
#include "sqlconstraint.h"

class QueryThread;
class MWidget;
class SearchPlugin;
class SearchDetailsWidget;

class QAbstractItemView;
class QTableView;
class QSqlRecord;
class QTextTable;

struct PreparedConstraints {
	QList<SearchPlugin*> lastSearchPlugins;
	QList<SqlConstraint> rawConstraints;
	QStringList strings;
	QList<QVariant> bindValues;
	QString table;
	QString order;
	bool usedDetailsWidget;
	bool needPersons;
	bool wish;
	bool projects;
	bool gifts;
	bool userNotNeeded;
	bool searchExternal;
	unsigned int actualUserId;
};

/**
  * The base class of all models holding media.
	* It is also used to handle the "All" types functionality
	* @todo If the QueryThread is used somewhere else than
	*       in load(), the thread should be setup
	*       at setupModel and not at the load function.
  */
class EXPORT_DLL MediaModel : public QAbstractItemModel
{
	Q_OBJECT
  public:
		/**
			* @brief Constructor for usage with subclasses.
			*
			* call setupModel() at the constructor of the subclass
			*/
    MediaModel ( QObject *parent = 0 );
		/**
			* @brief This constructor must be used, if this Model should be used directly
			*/
		MediaModel ( QueryThread *queryThread = 0, QObject *parent = 0 );
    virtual ~MediaModel();

    // own API
		QString group();
		QString name();
		/**
			* used by AddMedium to create a new model of the same type
			* to let the user search for a item he wants to add
			* @note should be reimplemented by the subclassed model, to create the right model,
			*       because the default implementation creates a pure MediaModel
			* @see AddMedium
			* @see InsertManualMedia
			*/		
		virtual MediaModel *createNewModel();
    /**
    	* creates an instance of a new medium
    	* but the medium isn't added to the model
    	*/
    virtual Medium *newMedium() const;
    /**
    	* adds a new medium to the model
    	* @attention Don't forget to emit the layoutAboutToBeChanged() and layoutChanged() signals
    	*/
    Medium *addEmptyMedium();
    bool addMedium ( unsigned int materialsId );
    bool addMedium ( Medium *medium );
    Medium *mediumFromIndex ( QModelIndex index ) const;
		QModelIndex indexFromMedium(Medium *medium);
		bool removeMedium(Medium *medium);
		bool removeMedium(QModelIndex index);
    bool store();
    /**
      * searches media in local database and with help of SearchPlugin's
      * @param userId > 0 searches local database for media of user userId
      * @param userId == 0 means default order (own media, other media, external)
      * @param userId -1 means all local media
			* @param userId -2 means external
      */
    virtual bool load(QList<SqlConstraint> constraints, QList<SearchPlugin*> plugins,
											int userId, SearchDetailsWidget *searchDetailsWidget = 0);
		/**
			* reloads the model with calling load with the parameters stored as last...
			* @todo the reloading isn't working with CD's (Books and Tracks is no problem)
			*/
		bool reload();
		/**
			* removes all items from the model
			*/
		bool clear();
		/**
			* this function set's the selection behavior (SelectRows) and modes (ExtendedSelection)
			* afterwards, it calls @see addItemDelegates
			* and if the view is an QTableView, @see hideAndResizeColums
			* is called
			*/
		virtual void configureView(QAbstractItemView *view);
    virtual void addItemDelegates(QAbstractItemView *view);
		void hideAndResizeColumns(QTableView *view);

    // QT API
    QVariant data ( const QModelIndex& index, int role = Qt::DisplayRole ) const;
    int columnCount ( const QModelIndex& parent = QModelIndex() ) const;
    int rowCount ( const QModelIndex& parent = QModelIndex() ) const;
    QModelIndex parent ( const QModelIndex& child ) const;
    QModelIndex index ( int row, int column, const QModelIndex& parent = QModelIndex() ) const;
    bool hasChildren ( const QModelIndex& parent = QModelIndex() ) const;
    bool setData ( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    QVariant headerData ( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
    virtual Qt::ItemFlags flags ( const QModelIndex& index ) const;
    void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );
		
	public slots:
		bool addMedium ( QSqlRecord record );
		
	protected slots:
		/**
			* this function is connect with the @see QueryThread::finished(bool) signal
			* If @p lastStepResult is false or the model does not have any rows,
			* the search should be continued under regards of the lastSearchConstraints
			*/
		void searchOtherUsers(bool lastStepResult);
		void searchSearchPlugins(bool lastStepResult);
		
		void errorByPass(QString msg);
		void warningByPass(QString msg);
		void informationByPass(QString msg);
		
		void showStatusWidgetByPass ( QWidget *widget );
    void removeStatusWidgetByPass ( QWidget *widget );
		
  protected:
		void setupModel(QueryThread *queryThread = 0);
		void connectMedium(Medium *object);
		
    QList<Medium*> media;
		QueryThread *thread;
		bool ownThread;
    /**
      * the condition to only load the handled media
      */
    SqlConstraint mediaCondition;
 		/**
			* a list of columns which should be hidden (index of column)
			*/
		QList<int> hideColumns;
		/**
			* the index of the column, which should be use the whole available
			* space after resizing all other columns to content
			*/
		int biggestColumn;
		QStringList headerList;

  private slots:
    void itemChanged(Medium *med);
		void setupThread();
		
	private:
		/**
			* Used to do all needed stuff to add an medium
			* to the list of media
			* (like emit layoutAboutToBeChanged, add it to the QList, ...)
			*/
		void doAddMedium(Medium *med);
		/**
			* it's the search mode id (this can be a userId, if > 0)
			*/
		int lastUserId;
		PreparedConstraints lastLoad;
		QString myName, myGroup;
		bool compatibilityMode;
    
  signals:
    void error(QString msg);
    void warning(QString msg);
    void information(QString msg);
		
		void showStatusWidget ( QWidget* widget );
    void removeStatusWidget ( QWidget *widget );
};

#endif // MEDIAMODEL_H
