/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mimetype.h"

#include "program.h"

#include <QString>
#include <QFileInfo>
#include <QSettings>
#include <QStringList>

MimeType::MimeType ( QString newName ) : MimeModelItem()
{
  myDefaultProgram = 0;
  setName ( newName );
}

MimeType::~MimeType()
{
	qDeleteAll(myPossiblePrograms);
}

void MimeType::setName ( QString newName )
{
  myName = newName;
}

QString MimeType::name() const
{
  return myName;
}

void MimeType::setSuffix ( QString suffix )
{
  mySuffix = suffix;
	if(!suffix.isEmpty())
		setPrefix(QString());
}

QString MimeType::suffix() const
{
  return mySuffix;
}

void MimeType::setPrefix ( QString prefix )
{
  myPrefix = prefix;
	if(!prefix.isEmpty())
		setSuffix(QString());
}

QString MimeType::prefix() const
{
  return myPrefix;
}

void MimeType::setDefaultProgram ( Program *newProgram )
{
  myDefaultProgram = newProgram;
	if(!newProgram)
		return;
	if(possiblePrograms().contains(newProgram))
		return;
	// because it is not listed as possible app, we add it
	setPossibleProgram(newProgram);
}

void MimeType::setDefaultProgram(QString newProgram)
{
	setDefaultProgram( new Program(newProgram) );
}

Program *MimeType::defaultProgram() const
{
  return myDefaultProgram;
}

void MimeType::setPossibleProgram ( QString possibleProgram )
{
  setPossibleProgram ( QFileInfo ( possibleProgram ) );
}

void MimeType::setPossibleProgram ( QFileInfo possibleProgram )
{
  Program *posApp = new Program ( possibleProgram.absoluteFilePath(), this );
  posApp->setProgram ( possibleProgram );
  setPossibleProgram ( posApp );
}

void MimeType::setPossibleProgram ( Program *possibleProgram )
{
	if(!possibleProgram)
		return;
  possibleProgram->setParent ( this );
  myPossiblePrograms << possibleProgram;
}

QList<Program*> MimeType::possiblePrograms() const
{
  return myPossiblePrograms;
}

int MimeType::rowCount() const
{
  return myPossiblePrograms.size();
}

bool MimeType::saveMimeType()
{
  QSettings qSettings;
  qSettings.beginGroup ( "MimeTypes" );

  // saving this mimetype
  QStringList save;
	save.append ( suffix() );
	save.append ( prefix() );

  if ( defaultProgram() ) {
    save.append ( defaultProgram()->file().absoluteFilePath() );

  } else {
    save.append ( "" );
  }

  QStringList possibleApps;

  foreach ( Program *app, possiblePrograms() ) {
    possibleApps.append ( app->file().filePath() );
  }

  save.append ( possibleApps.join ( "?" ) );
  qSettings.setValue ( name(), save );

  // appending this mimetype to the saved list
  QStringList savedTypes = qSettings.value ( "mimeTypes" ).toStringList();

  if ( savedTypes.indexOf ( name() ) == -1 ) {
    savedTypes.append ( name() );
    qSettings.setValue ( "mimeTypes", savedTypes );
  }

  qSettings.endGroup();

  return true;
}

bool MimeType::loadMimeType ( QString newName )
{
  if ( !newName.isEmpty() )
    setName ( newName );

  if ( name().isEmpty() )
    return false;

  QSettings qSettings;

  qSettings.beginGroup ( "MimeTypes" );

  // looking up list of stored mimeTypes
  QStringList savedTypes = qSettings.value ( "mimeTypes" ).toStringList();

  if ( savedTypes.indexOf ( name() ) == -1 ) {
    qSettings.endGroup();
    return false;
  }

  // it's in the list, so we can get data out of the settings
  QStringList savedData = qSettings.value ( name() ).toStringList();

  qSettings.endGroup();

  // it must be 3, because [0] - is suffix, [1] - is defaultProgram
  // [2] - are possible Programs joined with ?
  if ( savedData.size() != 4 )
    return false;

  setSuffix ( savedData[0] );
	setPrefix ( savedData[1] );

  setDefaultProgram ( new Program ( savedData[2] ) );

  QStringList possibleApps = savedData[3].split ( "?" );

  foreach ( QString posApp, possibleApps ) {
    setPossibleProgram ( posApp );
  }

  return true;
}

bool MimeType::operator== ( const MimeType &compareType ) const
{
  if ( name() == compareType.name() )
    return true;

  return false;
}

bool MimeType::operator!=(const MimeType& compareType) const
{
	return !operator==(compareType);
}

