/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mimetypemodel.h"

#include "mimetype.h"
#include "mimemodelitem.h"
#include "program.h"

#include <QSettings>
#include <QFileSystemModel>

#include <QDebug>

MimeTypeModel::MimeTypeModel ( QObject *parent ) : QAbstractItemModel ( parent )
{

}

MimeTypeModel::~MimeTypeModel()
{
	emit layoutAboutToBeChanged();
	qDeleteAll(mimeTypes);
	emit layoutChanged();
}

void MimeTypeModel::addMimeType ( MimeType *mimeType )
{
  emit layoutAboutToBeChanged();
  mimeTypes.append ( mimeType );
  emit layoutChanged();
}

MimeType *MimeTypeModel::getMimeType ( QString mimeType )
{
  for ( int i = 0; i < rowCount(); i++ ) {
    if ( mimeTypes[i]->name() == mimeType )
      return mimeTypes[i];
  }

  return 0;

//  MimeType *tmp = new MimeType(mimeType);
//  int index = mimeTypes.indexOf(tmp);
//  delete tmp;
//  if(index == -1)
//    return 0;
//  return mimeTypes[index];
}

MimeType *MimeTypeModel::getMimeType ( int row )
{
  if ( ( row < 0 ) || ( row >= mimeTypes.size() ) )
    return 0;

  return mimeTypes[row];
}

QModelIndex MimeTypeModel::index ( int row, int column, const QModelIndex &parent ) const
{
  if ( !parent.isValid() ) {
    // a mimetype is requested
    if ( ( 0 > row ) || ( row >= mimeTypes.size() ) )
      return QModelIndex();

    if ( ( 0 > column ) || ( column >= columnCount ( parent ) ) )
      return QModelIndex();

    return createIndex ( row, column, mimeTypes[row] );
  }

  if ( parent.column() != 0 )
    return QModelIndex();

  if ( ( 0 > row ) || ( row >= rowCount ( parent ) ) )
    return QModelIndex();

  MimeType *parentItem = mimeTypes[parent.row() ];

  if ( ( 0 > row ) || ( row >= rowCount ( parent ) ) )
    return QModelIndex();

  if ( ( 0 > column ) || ( column >= columnCount ( parent ) ) )
    return QModelIndex();

  return createIndex ( row, column, parentItem->possiblePrograms() [row] );
}

QModelIndex MimeTypeModel::parent ( const QModelIndex &index ) const
{
  // if it is a mimeType, the parent is the root node with invalid index QModelIndex()
  MimeModelItem *item = elementFromIndex ( index );

  if ( !item )
    return QModelIndex();

  MimeType *mimeType = dynamic_cast<MimeType*> ( item );

  if ( mimeType )
    return QModelIndex();

  // the application should know it's parent
  Program *application = dynamic_cast<Program*> ( item );

  if ( application ) {
    MimeType *parentItem = application->parent();

    if ( !parentItem )
      return QModelIndex();

    // find parent in mimeTypes-List
    int index = mimeTypes.indexOf ( parentItem );

    if ( index == -1 )
      return QModelIndex();

    return createIndex ( index, 0, parentItem );
  }

  return QModelIndex();
}

int MimeTypeModel::rowCount ( const QModelIndex &parent ) const
{
  MimeModelItem *item = elementFromIndex ( parent );

  if ( !item )
    return mimeTypes.size();

  MimeType *mimeType = dynamic_cast<MimeType*> ( item );

  if ( !mimeType )
    return 0;

  return mimeType->rowCount();
}

int MimeTypeModel::columnCount ( const QModelIndex &parent ) const
{
  MimeModelItem *item = elementFromIndex ( parent );

  if ( !item )
    return 4;

  MimeType *mimeType = dynamic_cast<MimeType*> ( item );

  if ( !mimeType )
    return 0;

  if ( mimeType->rowCount() > 0 )
    return 1;

  return 0;
}

QVariant MimeTypeModel::data ( const QModelIndex &index, int role ) const
{
  if ( role != Qt::DisplayRole )
    return QVariant();

  MimeModelItem *item = elementFromIndex ( index );

  if ( !item )
    return QVariant();

  MimeType *mimeType = dynamic_cast<MimeType*> ( item );

  if ( mimeType ) {
    switch ( index.column() ) {
      case 0:
        return mimeType->name();
      case 1:
        return mimeType->suffix();
      case 2:
	return mimeType->prefix();
      case 3:
        if ( mimeType->defaultProgram() )
          return mimeType->defaultProgram()->file().absoluteFilePath();

      default:
        return QVariant();
    }
  }

  Program *appItem = dynamic_cast<Program*> ( item );

  if ( appItem ) {
    return appItem->file().fileName();
  }

  return QVariant();
}

bool MimeTypeModel::hasChildren ( const QModelIndex &parent ) const
{
  MimeModelItem *item = elementFromIndex ( parent );

  if ( !item ) {
    if ( rowCount ( QModelIndex() ) > 0 )
      return true;

    return false;
  }

  MimeType *mimeItem = dynamic_cast<MimeType*> ( item );

  if ( mimeItem ) {
    if ( ( rowCount ( parent ) > 0 ) && ( parent.column() == 0 ) )
      return true;

    return false;
  }

  return false;
}

QVariant MimeTypeModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
  if ( role != Qt::DisplayRole )
    return QVariant();

  if ( orientation == Qt::Horizontal ) {
    switch ( section ) {
      case 0:
        return tr ( "Mime Type" );
      case 1:
        return tr ( "Suffix" );
      case 2:
        return tr ( "Prefix" );
      case 3:
        return tr ( "Default Program" );
      default:
        return QVariant();
    }
  }

  return QVariant();
}

MimeModelItem* MimeTypeModel::elementFromIndex ( const QModelIndex &index ) const
{
  if ( index.isValid() ) {
    return static_cast<MimeModelItem *> ( index.internalPointer() );

  } else {
    return 0;
  }
}

bool MimeTypeModel::loadModel()
{
  MimeType *item = 0;

  QSettings qSettings;
  qSettings.beginGroup ( "MimeTypes" );
  QStringList savedTypes = qSettings.value ( "mimeTypes" ).toStringList();
  foreach ( QString type, savedTypes ) {
    item = new MimeType ( type );

    if ( item->loadMimeType() )
      addMimeType ( item );
  }

  qSettings.endGroup();

  QFileSystemModel fileModel(this);
  QString rootPath = "/";
  #if defined(Q_OS_WIN32)
    rootPath = "c:/";
  #endif
  
  fileModel.setRootPath(rootPath);
  
  // adding default mime types
  item = getMimeType ( "application/pdf" );

  if ( !item ) {
    item = new MimeType ( "application/pdf" );
    item->setSuffix ( "pdf" );
    QStringList filters;
#if defined(Q_OS_WIN32)
    filters << "AcroRd32.exe";
    filters << "Acrobat.exe"; 
    // add adobe readers
    item->setPossibleProgram ( new Program ( "Adobe/Reader 8.0/Reader/AcroRd32.exe", item ) );
    item->setPossibleProgram ( new Program ( "Adobe/Reader 9.0/Reader/AcroRd32.exe", item ) );
    // add adobe acrobats
    item->setPossibleProgram ( new Program ( "Adobe/Acrobat 6.0/Acrobat/Acrobat.exe", item ) );
#else
    filters << "xpdf";
    filters << "okular";
    filters << "acroread";
    filters << "evince";
    filters << "skim";
    filters << "preview";
    item->setPossibleProgram ( new Program ( "xpdf", item ) );
    item->setPossibleProgram ( new Program ( "okular", item ) );
    item->setPossibleProgram ( new Program ( "acroread", item ) );
    item->setPossibleProgram ( new Program ( "evince", item ) );
    item->setPossibleProgram ( new Program ( "skim", item ) );
    item->setPossibleProgram ( new Program ( "preview", item ) ); // MAC OS X
#endif
    fileModel.setNameFilters(filters);
    
    for(int i = 0; i < fileModel.rowCount(); i++){
      QFileInfo info = fileModel.fileInfo(fileModel.index(i, 0));
      qDebug() << QString("Row %1: %2").arg(i).arg(info.absoluteFilePath());
    }
    
    addMimeType ( item );
  }

  // adding default mime types
  item = getMimeType ( "application/http" );

  if ( !item ) {
    item = new MimeType ( "application/http" );
    item->setPrefix( "http" );
#if defined(Q_OS_WIN32)
//     // add adobe readers
//     item->setPossibleProgram ( new Program ( "Adobe/Reader 8.0/Reader/AcroRd32.exe", item ) );
//     item->setPossibleProgram ( new Program ( "Adobe/Reader 9.0/Reader/AcroRd32.exe", item ) );
//     // add adobe acrobats
//     item->setPossibleProgram ( new Program ( "Adobe/Acrobat 6.0/Acrobat/Acrobat.exe", item ) );
#else
    item->setPossibleProgram ( new Program ( "konqueror", item ) );
    item->setPossibleProgram ( new Program ( "firefox", item ) );
#endif
    addMimeType ( item );
  }
  
  return true;
}

#include "mimetypemodel.moc"
