/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mimetypetests.h"

#include <mimetypemodel.h>
#include "modeltest.h"
#include <mimetype.h>
#include <program.h>

#include <QtTest/QtTest>
#include <QFileInfo>

void MimeTypeTests::initTestCase()
{
	// QSettings organization name und application name
  QCoreApplication::setOrganizationName ( "EngSaS" );
  QCoreApplication::setApplicationName ( "Testing" );
}

void MimeTypeTests::programmTests()
{
	QFileInfo app("/usr/bin/which");
	Program prog;
	
	// parent
	MimeType *parent = new MimeType();
	prog.setParent(parent);
	QCOMPARE(prog.parent(), parent);
	prog.setParent(0);
	delete parent;
	
	// path functions
	prog.setPath(app.absolutePath());
	QCOMPARE(prog.path(), app.absoluteDir());
	prog.setProgram(QFileInfo());
	QCOMPARE(prog.path(), QDir());
	prog.setPath(app.absoluteDir());
	QCOMPARE(prog.path(), app.absoluteDir());
	
	// program functions
	prog.setPath(app.absoluteDir());
	prog.setProgram(app.baseName());
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	prog.setProgram(app.baseName());
	prog.setPath(app.absoluteDir());
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	app.setFile("/d/e/f");
	prog.setProgram(app);
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	QCOMPARE(prog.path(), app.absoluteDir());
	
	// operator == and !=
	Program prog1;
	prog.setProgram(QFileInfo("/a/b/f"));
	prog1.setProgram(QFileInfo("/z/d/e/f"));
	QVERIFY(prog == prog1);
	prog1.setProgram(QFileInfo("/f/b/a"));
	QVERIFY(prog != prog1);
	
	Program *prog2 = new Program("/a/b/c");
	QList<Program*> lists;
	lists << prog2;
	QVERIFY(lists.contains(prog2));
	QVERIFY(!lists.contains(&prog1));
}


void MimeTypeTests::mimeTypeTests()
{
	QString name = "application/pdf";
	QString suffix = "pdf";
	QString prefix = "http";
	MimeType mimeType(name);
	QCOMPARE(mimeType.name(), name);
	QVERIFY(!mimeType.defaultProgram());
	mimeType.setSuffix(suffix);
	QCOMPARE(mimeType.suffix(), suffix);
	mimeType.setPrefix(prefix);
	QCOMPARE(mimeType.prefix(), prefix);
	QCOMPARE(mimeType.suffix(), QString());
	mimeType.setSuffix(suffix);
	QCOMPARE(mimeType.prefix(), QString());
	
	QString prog1 = "/a/b/c";
	QString prog2 = "/c/d";
	mimeType.setPossibleProgram(prog1);
	mimeType.setDefaultProgram(prog2);
	QCOMPARE(mimeType.rowCount(), 2);
	QCOMPARE(mimeType.possiblePrograms().size(), mimeType.rowCount());
	QCOMPARE(mimeType.defaultProgram()->file().absoluteFilePath(), prog2);
	mimeType.setDefaultProgram(prog1);
	QEXPECT_FAIL("", "Will fix in the next release", Continue);
	QCOMPARE(mimeType.rowCount(), 2);
	mimeType.saveMimeType();
	MimeType mime2;
	mime2.loadMimeType("application/pdf");
	QCOMPARE(mimeType, mime2);
	QCOMPARE(mimeType.defaultProgram()->file().absoluteFilePath(),
					 mime2.defaultProgram()->file().absoluteFilePath());
	QCOMPARE(mimeType.suffix(), mime2.suffix());
	QCOMPARE(mimeType.prefix(), mime2.prefix());
}

void MimeTypeTests::standardModelTests()
{
	MimeTypeModel *model = new MimeTypeModel(this);
	ModelTest *modeltest = new ModelTest(model, 0, this);
	QList<MimeType*> items;
	items << new MimeType();
	items << new MimeType("Test");
	items << new MimeType("Test 2");
	items[items.size() - 1]->setPossibleProgram("Bla1");
	items[items.size() - 1]->setPossibleProgram("Bla2");
	items[items.size() - 1]->setDefaultProgram(0);
	items << new MimeType("Test 3");
	items[items.size() - 1]->setPossibleProgram("Bla1");
	items[items.size() - 1]->setPossibleProgram("Bla2");
	items[items.size() - 1]->setPossibleProgram("Bla3");
	items[items.size() - 1]->setDefaultProgram(items[items.size() - 1]->possiblePrograms()[2]);
	
	foreach(MimeType *type, items)
		model->addMimeType(type);
	
	QCOMPARE(model->rowCount(), items.size());
	
	// remove a min from the beginning, the middle and the end
	if(model->rowCount() > 0)
		model->removeRow(0);
	
	if(model->rowCount() > 0)
		model->removeRow(model->rowCount() - 1);
	
	if(model->rowCount() > 2){
		int half = model->rowCount() / 2;
		model->removeRow(half);
	}
	
	delete modeltest;
// 	delete model;
}

void MimeTypeTests::mimeModelTests()
{
	MimeTypeModel *model = new MimeTypeModel(this);
	QVERIFY(model->loadModel());
	QCOMPARE(model->rowCount(), 2);
	
	// getMimeType(QString)
	QString getType = "application/pdf";
	MimeType *pdfType = model->getMimeType(getType);
	QVERIFY(pdfType);
	QCOMPARE(pdfType->name(), getType);
	
	// getMimeType(int)
	// first get row
	bool found = false;
	for(int i = 0; i < model->rowCount(); i++){
		if(model->data(model->index(i, 0, QModelIndex())).toString() == pdfType->name()){
			found = true;
			QCOMPARE(model->getMimeType(i), pdfType);
		}
	}
	QVERIFY(found);
	
	// data function on QModelIndex() (MimeType-Level)
	for(int i = 0; i < model->rowCount(); i++){
		MimeType *tmp = model->getMimeType(i);
		QVERIFY(tmp);
		QCOMPARE(tmp->name(),
						model->data(model->index(i,0, QModelIndex())).toString());
		QCOMPARE(tmp->suffix(),
						 model->data(model->index(i,1, QModelIndex())).toString());
		QCOMPARE(tmp->prefix(),
						 model->data(model->index(i,2, QModelIndex())).toString());
		QString result;
		if(tmp->defaultProgram()){
			result = tmp->defaultProgram()->file().absoluteFilePath();
		}
		QCOMPARE(result,
							model->data(model->index(i,3, QModelIndex())).toString());
		if(tmp->rowCount() > 0){
			// data function on Program-Level
			QCOMPARE(tmp->possiblePrograms()[0]->file().fileName(),
							 model->data(model->index(0, 0, model->index(i, 0, QModelIndex()))).toString()
			);
		}
	}
	
	delete model;
}

QTEST_MAIN(MimeTypeTests)

#include "mimetypetests.moc"
