/****************************************************************************
**
** Copyright (C) 2007 Trolltech ASA. All rights reserved.
** Copyright (C) 2010 Joachim Langenbach All rights reserved. (QDebug and verbose part)
**
** This file is part of the Qt Concurrent project on Trolltech Labs.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <QtGui>

#include "modeltest.h"

#include <QtTest/QtTest>
#include <QDebug>

Q_DECLARE_METATYPE(QModelIndex)

/*!
    Connect to all of the models signals.  Whenever anything happens recheck everything.
*/
ModelTest::ModelTest(QAbstractItemModel *_model, int verbose, QObject *parent) : QObject(parent), model(_model), fetchingMore(false)
{
    verboseLevel = verbose;
    Q_ASSERT(model);

    connect(model, SIGNAL(columnsAboutToBeInserted(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(columnsAboutToBeRemoved(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(columnsInserted(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(columnsRemoved(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(headerDataChanged(Qt::Orientation, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(layoutAboutToBeChanged ()), this, SLOT(runAllTests()));
    connect(model, SIGNAL(layoutChanged ()), this, SLOT(runAllTests()));
    connect(model, SIGNAL(modelReset ()), this, SLOT(runAllTests()));
    connect(model, SIGNAL(rowsAboutToBeInserted(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(rowsAboutToBeRemoved(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(rowsInserted(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));
    connect(model, SIGNAL(rowsRemoved(const QModelIndex &, int, int)),
            this, SLOT(runAllTests()));

    // Special checks for inserting/removing
    connect(model, SIGNAL(layoutAboutToBeChanged()),
            this, SLOT(layoutAboutToBeChanged()));
    connect(model, SIGNAL(layoutChanged()),
            this, SLOT(layoutChanged()));

    connect(model, SIGNAL(rowsAboutToBeInserted(const QModelIndex &, int, int)),
            this, SLOT(rowsAboutToBeInserted(const QModelIndex &, int, int)));
    connect(model, SIGNAL(rowsAboutToBeRemoved(const QModelIndex &, int, int)),
            this, SLOT(rowsAboutToBeRemoved(const QModelIndex &, int, int)));
    connect(model, SIGNAL(rowsInserted(const QModelIndex &, int, int)),
            this, SLOT(rowsInserted(const QModelIndex &, int, int)));
    connect(model, SIGNAL(rowsRemoved(const QModelIndex &, int, int)),
            this, SLOT(rowsRemoved(const QModelIndex &, int, int)));

    runAllTests();
}

void ModelTest::runAllTests()
{
    if (fetchingMore)
        return;
    nonDestructiveBasicTest();
    rowCount();
    columnCount();
    hasIndex();
    index();
    parent();
    data();
}

/*!
    nonDestructiveBasicTest tries to call a number of the basic functions (not all)
    to make sure the model doesn't outright segfault, testing the functions that makes sense.
*/
void ModelTest::nonDestructiveBasicTest()
{
    if(verboseLevel > 0)
      qDebug() << "non destructive basic tests ...";
      
    if(verboseLevel > 1)
      qDebug() << "    model->buddy(QModelIndex()) == QModelIndex()";
    QVERIFY(model->buddy(QModelIndex()) == QModelIndex());
    
    if(verboseLevel > 1)
      qDebug() << "    model->canFetchMore(QModelIndex())";
    model->canFetchMore(QModelIndex());
    
    if(verboseLevel > 1)
      qDebug() << "    model->columnCount(QModelIndex()) >= 0";
    QVERIFY(model->columnCount(QModelIndex()) >= 0);
    
    if(verboseLevel > 1)
      qDebug() << "    model->data(QModelIndex()) == QVariant()";
    QVERIFY(model->data(QModelIndex()) == QVariant());
    
    fetchingMore = true;
    if(verboseLevel > 1)
      qDebug() << "    model->fetchMore(QModelIndex())";
    model->fetchMore(QModelIndex());
    fetchingMore = false;
    
    if(verboseLevel > 1)
      qDebug() << "    Qt::ItemFlags flags = model->flags(QModelIndex())";
    Qt::ItemFlags flags = model->flags(QModelIndex());

//     QVERIFY(flags == Qt::ItemIsDropEnabled || flags == 0);
    if(verboseLevel > 1)
      qDebug() << "    model->hasChildren(QModelIndex())";
    model->hasChildren(QModelIndex());
    
    if(verboseLevel > 1)
      qDebug() << "    model->hasIndex(0, 0)";
    model->hasIndex(0, 0);
    
    if(verboseLevel > 1)
      qDebug() << "    model->headerData(0, Qt::Horizontal)";
    model->headerData(0, Qt::Horizontal);
    
    if(verboseLevel > 1)
      qDebug() << "    model->index(0, 0)";
    model->index(0, 0);
    
    if(verboseLevel > 1)
      qDebug() << "    model->index(-1, -1) == QModelIndex()";
    QVERIFY(model->index(-1, -1) == QModelIndex());
    
    if(verboseLevel > 1)
      qDebug() << "    model->itemData(QModelIndex())";
    model->itemData(QModelIndex());
    QVariant cache;
    if(verboseLevel > 1)
      qDebug() << "    model->match(QModelIndex(), -1, cache)";
    model->match(QModelIndex(), -1, cache);
    if(verboseLevel > 1)
      qDebug() << "    model->mimeTypes()";
    model->mimeTypes();
    if(verboseLevel > 1)
      qDebug() << "    model->parent(QModelIndex()) == QModelIndex()";
    QVERIFY(model->parent(QModelIndex()) == QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->rowCount() >= 0";
    QVERIFY(model->rowCount() >= 0);
    
    QVariant variant;
    if(verboseLevel > 1)
      qDebug() << "    model->setData(QModelIndex(), variant, -1)";
    model->setData(QModelIndex(), variant, -1);
    if(verboseLevel > 1)
      qDebug() << "    model->setHeaderData(-1, Qt::Horizontal, QVariant())";
    model->setHeaderData(-1, Qt::Horizontal, QVariant());
    if(verboseLevel > 1)
      qDebug() << "    model->setHeaderData(0, Qt::Horizontal, QVariant())";
    model->setHeaderData(0, Qt::Horizontal, QVariant());
    if(verboseLevel > 1)
      qDebug() << "    model->setHeaderData(999999, Qt::Horizontal, QVariant())";
    model->setHeaderData(999999, Qt::Horizontal, QVariant());
    
    QMap<int, QVariant> roles;
    if(verboseLevel > 1)
      qDebug() << "    model->sibling(0, 0, QModelIndex())";
    model->sibling(0, 0, QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->span(QModelIndex())";
    model->span(QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->supportedDropActions()";
    model->supportedDropActions();
    
    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Tests model's implementation of QAbstractItemModel::rowCount() and hasChildren()

    Models that are dynamically populated are not as fully tested here.
 */
void ModelTest::rowCount()
{
  if(verboseLevel > 0){
    qDebug() << "testing implementation of rowCount() und hasChildren() ...";
    qDebug() << "      (Be aware: dynamically populated models are not tested very well)";
  }

    // check top row
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex topIndex = model->index(0, 0, QModelIndex())";
    QModelIndex topIndex = model->index(0, 0, QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->rowCount(topIndex)";
    int rows = model->rowCount(topIndex);
    QVERIFY(rows >= 0);
    if (rows > 0){
      if(verboseLevel > 1)
        qDebug() << "    model->hasChildren(topIndex) == true";
      QVERIFY(model->hasChildren(topIndex) == true);
    }

    if(verboseLevel > 1)
      qDebug() << "    secondLevelIndex = model->index(0, 0, topIndex)";
    QModelIndex secondLevelIndex = model->index(0, 0, topIndex);
    if (secondLevelIndex.isValid()) { // not the top level
        // check a row count where parent is valid
      if(verboseLevel > 1)
        qDebug() << "    model->rowCount(secondLevelIndex)";
        rows = model->rowCount(secondLevelIndex);
        QVERIFY(rows >= 0);
        if (rows > 0){
          if(verboseLevel > 1)
            qDebug() << "    model->hasChildren(secondLevelIndex) == true";
          QVERIFY(model->hasChildren(secondLevelIndex) == true);
        }
    }

    // The models rowCount() is tested more extensively in checkChildren(),
    // but this catches the big mistakes

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Tests model's implementation of QAbstractItemModel::columnCount() and hasChildren()
 */
void ModelTest::columnCount()
{
  if(verboseLevel > 0)
    qDebug() << "testing columnCount() and hasChildren() ...";

    // check top row
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex topIndex = model->index(0, 0, QModelIndex())";
    QModelIndex topIndex = model->index(0, 0, QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->columnCount(topIndex) >= 0";
    QVERIFY(model->columnCount(topIndex) >= 0);

    // check a column count where parent is valid
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex childIndex = model->index(0, 0, topIndex)";
    QModelIndex childIndex = model->index(0, 0, topIndex);
    if (childIndex.isValid()){
      if(verboseLevel > 1)
        qDebug() << "    model->columnCount(childIndex) >= 0";
      QVERIFY(model->columnCount(childIndex) >= 0);
    }

    // columnCount() is tested more extensively in checkChildren(),
    // but this catches the big mistakes

  if(verboseLevel > 0)
    qDebug() << "  passed";
}

/*!
    Tests model's implementation of QAbstractItemModel::hasIndex()
 */
void ModelTest::hasIndex()
{
  if(verboseLevel > 0)
    qDebug() << "testing hasIndex() ...";

    // Make sure that invalid values returns an invalid index
  if(verboseLevel > 1)
    qDebug() << "    model->hasIndex(-2, -2) == false";
    QVERIFY(model->hasIndex(-2, -2) == false);
    if(verboseLevel > 1)
      qDebug() << "    model->hasIndex(-2, 0) == false";
    QVERIFY(model->hasIndex(-2, 0) == false);
    if(verboseLevel > 1)
      qDebug() << "    model->hasIndex(0, -2) == false";
    QVERIFY(model->hasIndex(0, -2) == false);

    int rows = model->rowCount();
    int columns = model->columnCount();

    // check out of bounds
    if(verboseLevel > 1)
      qDebug() << "    model->hasIndex(rows, columns) == false";
    QVERIFY(model->hasIndex(rows, columns) == false);
    if(verboseLevel > 1)
      qDebug() << "    model->hasIndex(rows + 1, columns + 1) == false";
    QVERIFY(model->hasIndex(rows + 1, columns + 1) == false);

    if (rows > 0){
      if(verboseLevel > 1)
        qDebug() << "    model->hasIndex(0, 0) == true";
        QVERIFY(model->hasIndex(0, 0) == true);
    }

    // hasIndex() is tested more extensively in checkChildren(),
    // but this catches the big mistakes

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Tests model's implementation of QAbstractItemModel::index()
 */
void ModelTest::index()
{
  if(verboseLevel > 0)
    qDebug() << "testing index() ...";

    // Make sure that invalid values returns an invalid index
    if(verboseLevel > 1)
      qDebug() << "    model->index(-2, -2) == QModelIndex()";
    QVERIFY(model->index(-2, -2) == QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->index(-2, 0) == QModelIndex()";
    QVERIFY(model->index(-2, 0) == QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->index(0, -2) == QModelIndex()";
    QVERIFY(model->index(0, -2) == QModelIndex());

    int rows = model->rowCount();
    int columns = model->columnCount();

    if (rows == 0)
        return;

    // Catch off by one errors
    if(verboseLevel > 1)
      qDebug() << "    model->index(rows, columns) == QModelIndex()";
    QVERIFY(model->index(rows, columns) == QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->index(0, 0).isValid() == true";
    QVERIFY(model->index(0, 0).isValid() == true);

    // Make sure that the same index is *always* returned
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex a = model->index(0, 0)";
    QModelIndex a = model->index(0, 0);
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex b = model->index(0, 0)";
    QModelIndex b = model->index(0, 0);
    if(verboseLevel > 1)
      qDebug() << "    a == b";
    QVERIFY(a == b);

    // index() is tested more extensively in checkChildren(),
    // but this catches the big mistakes

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Tests model's implementation of QAbstractItemModel::parent()
 */
void ModelTest::parent()
{

  if(verboseLevel > 0)
    qDebug() << "testing parent() ...";

    // Make sure the model wont crash and will return an invalid QModelIndex
    // when asked for the parent of an invalid index.
  if(verboseLevel > 1)
    qDebug() << "    model->parent(QModelIndex()) == QModelIndex()";
    QVERIFY(model->parent(QModelIndex()) == QModelIndex());

    if (model->rowCount() == 0)
        return;

    // Column 0                | Column 1    |
    // QModelIndex()           |             |
    //    \- topIndex          | topIndex1   |
    //         \- childIndex   | childIndex1 |

    // Common error test #1, make sure that a top level index has a parent
    // that is a invalid QModelIndex.
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex topIndex = model->index(0, 0, QModelIndex())";
    QModelIndex topIndex = model->index(0, 0, QModelIndex());
    if(verboseLevel > 1)
      qDebug() << "    model->parent(topIndex) == QModelIndex()";
    QVERIFY(model->parent(topIndex) == QModelIndex());

    // Common error test #2, make sure that a second level index has a parent
    // that is the first level index.
    if (model->rowCount(topIndex) > 0) {
      if(verboseLevel > 1)
        qDebug() << "    QModelIndex childIndex = model->index(0, 0, topIndex)";
        QModelIndex childIndex = model->index(0, 0, topIndex);
        if(verboseLevel > 1)
          qDebug() << "    model->parent(childIndex) == topIndex)";
        QVERIFY(model->parent(childIndex) == topIndex);
    }

    // Common error test #3, the second column should NOT have the same children
    // as the first column in a row.
    // Usually the second column shouldn't have children.
    if(verboseLevel > 1)
      qDebug() << "    QModelIndex topIndex1 = model->index(0, 1, QModelIndex())";
    QModelIndex topIndex1 = model->index(0, 1, QModelIndex());
    if (model->rowCount(topIndex1) > 0) {
      if(verboseLevel > 1)
        qDebug() << "    QModelIndex childIndex = model->index(0, 0, topIndex)";
        QModelIndex childIndex = model->index(0, 0, topIndex);
        if(verboseLevel > 1)
          qDebug() << "    QModelIndex childIndex1 = model->index(0, 0, topIndex1)";
        QModelIndex childIndex1 = model->index(0, 0, topIndex1);
        if(verboseLevel > 1)
          qDebug() << "    childIndex != childIndex1 (the second column should NOT have the same children as the first column in a row. Usually the second column shouldn't have children)";
        QVERIFY(childIndex != childIndex1);
    }

    // Full test, walk n levels deep through the model making sure that all
    // parent's children correctly specify their parent.
    if(verboseLevel > 0)
      qDebug() << "  testing childrens";
    checkChildren(QModelIndex());
    if(verboseLevel > 0)
      qDebug() << "    passed";

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Called from the parent() test.

    A model that returns an index of parent X should also return X when asking
    for the parent of the index.

    This recursive function does pretty extensive testing on the whole model in an
    effort to catch edge cases.

    This function assumes that rowCount(), columnCount() and index() already work.
    If they have a bug it will point it out, but the above tests should have already
    found the basic bugs because it is easier to figure out the problem in
    those tests then this one.
 */
void ModelTest::checkChildren(const QModelIndex &parent, int currentDepth)
{
    QString spaces = "    ";
    for(int i = 0; i < currentDepth; i++){
      spaces += "  ";
    }

    // First just try walking back up the tree.
    if(verboseLevel > 1)
      qDebug() << spaces +"First just try walking back up the tree";
    QModelIndex p = parent;
    while (p.isValid())
        p = p.parent();

    // For models that are dynamically populated
    if (model->canFetchMore(parent)) {
        if(verboseLevel > 1)
          qDebug() << spaces +"model->fetchMore(parent)";
        fetchingMore = true;
        model->fetchMore(parent);
        fetchingMore = false;
    }

    int rows = model->rowCount(parent);
    int columns = model->columnCount(parent);

    if (rows > 0){
      if(verboseLevel > 1)
        qDebug() << spaces +"model->hasChildren(parent)";
      QVERIFY(model->hasChildren(parent));
    }

    // Some further testing against rows(), columns(), and hasChildren()
    QVERIFY(rows >= 0);
    QVERIFY(columns >= 0);
    if (rows > 0){
      if(verboseLevel > 1)
        qDebug() << spaces +"model->hasChildren(parent) == true";
        QVERIFY(model->hasChildren(parent) == true);
    }

    //qDebug() << "parent:" << model->data(parent).toString() << "rows:" << rows
    //         << "columns:" << columns << "parent column:" << parent.column();

    if(verboseLevel > 1)
      qDebug() << spaces +"model->hasIndex(rows + 1, 0, parent) == false";
    QVERIFY(model->hasIndex(rows + 1, 0, parent) == false);
    for (int r = 0; r < rows; ++r) {
        if (model->canFetchMore(parent)) {
            if(verboseLevel > 1)
              qDebug() << spaces +"model->fetchMore(parent)";
            fetchingMore = true;
            model->fetchMore(parent);
            fetchingMore = false;
        }
        if(verboseLevel > 1)
          qDebug() << spaces +"model->hasIndex(r, columns + 1, parent) == false";
        QVERIFY(model->hasIndex(r, columns + 1, parent) == false);
        for (int c = 0; c < columns; ++c) {
          if(verboseLevel > 1)
            qDebug() << QString("%1QModelIndex index = model->index(%2, %3, parent) parent: %4, %5")
              .arg(spaces)
              .arg(r)
              .arg(c)
              .arg(parent.row())
              .arg(parent.column());
            QVERIFY(model->hasIndex(r, c, parent) == true);
            QModelIndex index = model->index(r, c, parent);
            // rowCount() and columnCount() said that it existed...
            if(verboseLevel > 1)
              qDebug() << spaces +"index.isValid() == true";
            QVERIFY(index.isValid() == true);

            // index() should always return the same index when called twice in a row
            if(verboseLevel > 1)
              qDebug() << spaces +"index() should always return the same index when called twice in a row";
            QModelIndex modifiedIndex = model->index(r, c, parent);
            QVERIFY(index == modifiedIndex);

            // Make sure we get the same index if we request it twice in a row
            if(verboseLevel > 1)
              qDebug() << spaces +"Make sure we get the same index if we request it twice in a row";
            QModelIndex a = model->index(r, c, parent);
            QModelIndex b = model->index(r, c, parent);
            QVERIFY(a == b);

            // Some basic checking on the index that is returned
            if(verboseLevel > 1)
              qDebug() << spaces +"Some basic checking on the index that is returned";
            QVERIFY(index.model() == model);
            QVERIFY(index.row() == r);
            QVERIFY(index.column() == c);
            // While you can technically return a QVariant usually this is a sign
            // of an bug in data()  Disable if this really is ok in your model.
            //QVERIFY(model->data(index, Qt::DisplayRole).isValid() == true);

            // If the next test fails here is some somewhat useful debug you play with.
            /*
            if (model->parent(index) != parent) {
                qDebug() << r << c << currentDepth << model->data(index).toString()
                         << model->data(parent).toString();
                qDebug() << index << parent << model->parent(index);
                // And a view that you can even use to show the model.
                //QTreeView view;
                //view.setModel(model);
                //view.show();
            }*/

            // Check that we can get back our real parent.
            if(verboseLevel > 1)
              qDebug() << spaces +"Check that we can get back our real parent";
            QModelIndex p = model->parent(index);
            //qDebug() << "child:" << index;
            //qDebug() << p;
            //qDebug() << parent;
            QVERIFY(model->parent(index) == parent);

            // recursively go down the children
            if(verboseLevel > 1)
              qDebug() << spaces +"recursively go down the children";
            if (model->hasChildren(index) && currentDepth < 10 ) {
                //qDebug() << r << c << "has children" << model->rowCount(index);
                checkChildren(index, ++currentDepth);
            }/* else { if (currentDepth >= 10) qDebug() << "checked 10 deep"; };*/

            // make sure that after testing the children that the index doesn't change.
            if(verboseLevel > 1)
              qDebug() << spaces +"make sure that after testing the children that the index doesn't change";
            QModelIndex newerIndex = model->index(r, c, parent);
            QVERIFY(index == newerIndex);
        }
    }
}

/*!
    Tests model's implementation of QAbstractItemModel::data()
 */
void ModelTest::data()
{
  if(verboseLevel > 0)
    qDebug() << "testing data() ...";

    // Invalid index should return an invalid qvariant
    if(verboseLevel > 1)
      qDebug() << "    Invalid index should return an invalid qvariant";
    QVERIFY(!model->data(QModelIndex()).isValid());

    if (model->rowCount() == 0)
        return;

    // A valid index should have a valid QVariant data
    if(verboseLevel > 1)
      qDebug() << "    A valid index should have a valid QVariant data";
    QVERIFY(model->index(0, 0).isValid());

    // shouldn't be able to set data on an invalid index
    if(verboseLevel > 1)
      qDebug() << "    shouldn't be able to set data on an invalid index";
    QVERIFY(model->setData(QModelIndex(), QLatin1String("foo"), Qt::DisplayRole) == false);

    // General Purpose roles that should return a QString
    if(verboseLevel > 1)
      qDebug() << "    First just try walking back up the tree";
    QVariant variant = model->data(model->index(0, 0), Qt::ToolTipRole);
    if (variant.isValid()) {
        QVERIFY(qVariantCanConvert<QString>(variant));
    }
    variant = model->data(model->index(0, 0), Qt::StatusTipRole);
    if (variant.isValid()) {
        QVERIFY(qVariantCanConvert<QString>(variant));
    }
    variant = model->data(model->index(0, 0), Qt::WhatsThisRole);
    if (variant.isValid()) {
        QVERIFY(qVariantCanConvert<QString>(variant));
    }

    // General Purpose roles that should return a QSize
    if(verboseLevel > 1)
      qDebug() << "    General Purpose roles that should return a QSize";
    variant = model->data(model->index(0, 0), Qt::SizeHintRole);
    if (variant.isValid()) {
        QVERIFY(qVariantCanConvert<QSize>(variant));
    }

    // General Purpose roles that should return a QFont
    if(verboseLevel > 1)
      qDebug() << "    General Purpose roles that should return a QFont";
    QVariant fontVariant = model->data(model->index(0, 0), Qt::FontRole);
    if (fontVariant.isValid()) {
        QVERIFY(qVariantCanConvert<QFont>(fontVariant));
    }

    // Check that the alignment is one we know about
    if(verboseLevel > 1)
      qDebug() << "    Check that the alignment is one we know about";
    QVariant textAlignmentVariant = model->data(model->index(0, 0), Qt::TextAlignmentRole);
    if (textAlignmentVariant.isValid()) {
        int alignment = textAlignmentVariant.toInt();
        QVERIFY(alignment == Qt::AlignLeft ||
                 alignment == Qt::AlignRight ||
                 alignment == Qt::AlignHCenter ||
                 alignment == Qt::AlignJustify ||
                 alignment == Qt::AlignTop ||
                 alignment == Qt::AlignBottom ||
                 alignment == Qt::AlignVCenter ||
                 alignment == Qt::AlignCenter ||
                 alignment == Qt::AlignAbsolute ||
                 alignment == Qt::AlignLeading ||
                 alignment == Qt::AlignTrailing);
    }

    // General Purpose roles that should return a QColor
    if(verboseLevel > 1)
      qDebug() << "    General Purpose roles that should return a QColor";
    QVariant colorVariant = model->data(model->index(0, 0), Qt::BackgroundColorRole);
    if (colorVariant.isValid()) {
        QVERIFY(qVariantCanConvert<QColor>(colorVariant));
    }

    colorVariant = model->data(model->index(0, 0), Qt::TextColorRole);
    if (colorVariant.isValid()) {
        QVERIFY(qVariantCanConvert<QColor>(colorVariant));
    }

    // Check that the "check state" is one we know about.
    if(verboseLevel > 1)
      qDebug() << "    Check that the \"check state\" is one we know about";
    QVariant checkStateVariant = model->data(model->index(0, 0), Qt::CheckStateRole);
    if (checkStateVariant.isValid()) {
        int state = checkStateVariant.toInt();
        QVERIFY(state == Qt::Unchecked ||
                 state == Qt::PartiallyChecked ||
                 state == Qt::Checked);
    }
    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Store what is about to be inserted to make sure it actually happens

    \sa rowsInserted()
 */
void ModelTest::rowsAboutToBeInserted(const QModelIndex &parent, int start, int end)
{
    Q_UNUSED(end);
    Changing c;
    c.parent = parent;
    c.oldSize = model->rowCount(parent);
    c.last = model->data(model->index(start - 1, 0, parent));
    c.next = model->data(model->index(start, 0, parent));
    insert.push(c);
}

/*!
    Confirm that what was said was going to happen actually did

    \sa rowsAboutToBeInserted()
 */
void ModelTest::rowsInserted(const QModelIndex & parent, int start, int end)
{
  if(verboseLevel > 0)
    qDebug() << "testing the insertion of rows ...";

    Changing c = insert.pop();
    QVERIFY(c.parent == parent);
    QVERIFY(c.oldSize + (end - start + 1) == model->rowCount(parent));
    QVERIFY(c.last == model->data(model->index(start - 1, 0, c.parent)));
    /*
    if (c.next != model->data(model->index(end + 1, 0, c.parent))) {
        qDebug() << start << end;
        for (int i=0; i < model->rowCount(); ++i)
            qDebug() << model->index(i, 0).data().toString();
        qDebug() << c.next << model->data(model->index(end + 1, 0, c.parent));
    }
    */
    QVERIFY(c.next == model->data(model->index(end + 1, 0, c.parent)));

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

void ModelTest::layoutAboutToBeChanged()
{
    for (int i = 0; i < qBound(0, model->rowCount(), 100); ++i)
        changing.append(QPersistentModelIndex(model->index(i, 0)));
}

void ModelTest::layoutChanged()
{
    if(verboseLevel > 0)
      qDebug() << "testing the layout change ...";

    for (int i = 0; i < changing.count(); ++i) {
        QPersistentModelIndex p = changing[i];
        QVERIFY(p == model->index(p.row(), p.column(), p.parent()));
    }
    changing.clear();

    if(verboseLevel > 0)
      qDebug() << "  passed";
}

/*!
    Store what is about to be inserted to make sure it actually happens

    \sa rowsRemoved()
 */
void ModelTest::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
{
    Changing c;
    c.parent = parent;
    c.oldSize = model->rowCount(parent);
    c.last = model->data(model->index(start - 1, 0, parent));
    c.next = model->data(model->index(end + 1, 0, parent));
    remove.push(c);
}

/*!
    Confirm that what was said was going to happen actually did

    \sa rowsAboutToBeRemoved()
 */
void ModelTest::rowsRemoved(const QModelIndex & parent, int start, int end)
{
  if(verboseLevel > 0)
    qDebug() << "testing the removing of rows ...";

    Changing c = remove.pop();
    QVERIFY(c.parent == parent);
    QVERIFY(c.oldSize - (end - start + 1) == model->rowCount(parent));
    QVERIFY(c.last == model->data(model->index(start - 1, 0, c.parent)));
    QVERIFY(c.next == model->data(model->index(start, 0, c.parent)));

    if(verboseLevel > 1)
      qDebug() << "  passed";
}

#include "modeltest.moc"
