/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "tracksmodel.h"

#include "track.h"
#include <delegateforeignkey.h>
#include <delegateuser.h>
#include <msqldatabase.h>
#include "cds.h"

#include <QAbstractItemView>
#include <QSqlQuery>
#include <QProgressBar>

TracksModel::TracksModel ( QueryThread *queryThread, QObject* parent ) : MediaModel ( parent )
{
	myAlbumMode = false;
	myCollectionMode = false;
	setupModel(queryThread);
}

MediaModel* TracksModel::createNewModel()
{
	TracksModel *model = new TracksModel(0);
// 	if(myAlbumMode)
// 		model->setAlbumMode(true);
	return model;
}

bool TracksModel::addMedium(unsigned int materialsId)
{
    return MediaModel::addMedium(materialsId);
}

bool TracksModel::addMedium(Medium* medium)
{
	Track *track = qobject_cast<Track*>(medium);
	if(!track)
		return false;

  if(!MediaModel::addMedium(medium))
		return false;

	if(albumMode())
		track->setAlbumMode(true);
	if(collectionMode())
		track->setCollectionMode(true);
	return true;
}

Medium *TracksModel::newMedium() const
{
	Track *track = new Track();
	if(albumMode())
		track->setAlbumMode(true);
	if(collectionMode())
		track->setCollectionMode(true);
  return track;
}

bool TracksModel::store(Medium* parent, bool ignoreRemoved)
{
	if(!parent)
		return false;
	if(albumMode())
		return storeAlbumMode(parent, ignoreRemoved);
	if(collectionMode())
		return storeCollectionMode(parent, ignoreRemoved);
	return false;
}

bool TracksModel::loadTracks(Medium* parent)
{
	if(parent->id() < 1)
		return false;
	
	if((!albumMode()) && (!collectionMode()))
		return false;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		if( !db.open() )
			return false;
	
	QSqlQuery query;
	if(!query.exec(QString("SELECT * FROM materialhasmaterials WHERE materialsId=%1").arg(parent->id()))){
		if(closeDB)
			db.close();
		return false;
	}
	
	while(query.next()){
		Track *track = qobject_cast<Track*>(addEmptyMedium());
		if(!track){
			if(closeDB)
				db.close();
			return false;
		}

		if(!track->select(query.record().value("hasMaterialsId").toUInt())){
			if(closeDB)
				db.close();
			return false;
		}
		
		QSqlQuery extraQuery;
		if(albumMode()){
			// add disc and track number
			extraQuery.exec(QString("SELECT * FROM %1_ondisc WHERE materialhasmaterialsId=%2").arg(CDs::group()).arg(query.record().value("id").toUInt()));
			if(!extraQuery.first()){
				if(closeDB)
					db.close();
				return false;
			}
			track->setDisc(extraQuery.record().value("disc").toUInt());
			track->setTrackNumber(extraQuery.record().value("number").toUInt());
		}
		else if(collectionMode()){
			// add track number
			extraQuery.exec(QString("SELECT * FROM %1_trackhastracks WHERE materialhasmaterialsId=%2").arg(CDs::group()).arg(query.record().value("id").toUInt()));
			if(!extraQuery.first()){
				if(closeDB)
					db.close();
				return false;
			}
			track->setTrackNumber(extraQuery.record().value("number").toUInt());
		}
	}
	
	if(closeDB)
		db.close();
	return true;
}

void TracksModel::addItemDelegates(QAbstractItemView* view)
{
  if(!view)
    return;
  
	if(!myAlbumMode){
		Medium *medium = newMedium();
		if(!medium)
			return;
		
		QStringList attrs = medium->supportedAttributes();
		view->setItemDelegateForColumn(medium->attrs("owner").column, new DelegateUser(view));
		delete medium;
	}
}

Qt::ItemFlags TracksModel::flags ( const QModelIndex& index ) const
{
	if(albumMode()){
		switch(index.column()){
			case 0:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
			case 1:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
		}
	}
	else if(collectionMode()){
		switch(index.column()){
			case 0:
				return MediaModel::flags ( index ) | Qt::ItemIsEditable;
		}
	}
	return MediaModel::flags ( index );
}

void TracksModel::setAlbumMode ( bool enable )
{
	myAlbumMode = enable;
	if(myAlbumMode)
		setCollectionMode(false);
	
	// change headerList
	int last = headerList.size();
	Medium *med = newMedium();
	if ( med )
    headerList = med->supportedAttributesTrans();
	delete med;
	emit headerDataChanged(Qt::Horizontal, 0, last);
	
	foreach(Medium *medium, media){
		Track *track = qobject_cast<Track*>(medium);
		if(track){
			track->setAlbumMode(myAlbumMode);
		}
	}
}

bool TracksModel::albumMode() const
{
	return myAlbumMode;
}

void TracksModel::setCollectionMode(bool enable)
{
	myCollectionMode = enable;
	if(myCollectionMode)
		setAlbumMode(false);
	
	// change headerList
	int last = headerList.size();
	Medium *med = newMedium();
	if ( med )
    headerList = med->supportedAttributesTrans();
	delete med;
	emit headerDataChanged(Qt::Horizontal, 0, last);
	
	emit layoutAboutToBeChanged();
	foreach(Medium *medium, media){
		Track *track = qobject_cast<Track*>(medium);
		if(track)
			track->setCollectionMode(myCollectionMode);
	}
	emit layoutChanged();
}

bool TracksModel::collectionMode() const
{
	return myCollectionMode;
}

bool TracksModel::storeAlbumMode(Medium* parent, bool ignoreRemoved)
{
	if(rowCount() < 1)
		return true;
	
	if(parent->id() < 1)
		return false;
	
	MSqlDatabase db = MSqlDatabase::database();
	if( !db.isOpen() )
		return false;
	
	QProgressBar *progBar = new QProgressBar();
	progBar->setRange(0, rowCount());
	emit showStatusWidget(progBar);
	
	QSqlQuery query;
	foreach(Medium *med, media){
		Track *track = qobject_cast<Track*>(med);
		if(!track){
			emit removeStatusWidget(progBar);
			return false;
		}
		// insert our own genre
		if(track->genre() == 0){
			track->setGenre(parent->genre(), false);
			track->setGenre(parent->genreName(), false);
		}
		// insert own manufacturer
		if(track->manufacturerName().isEmpty()){
			track->setManufacturer(parent->manufacturer(), false);
			track->setManufacturer(parent->manufacturerName(), false);
		}
		// if ignoreRemoved is set to true, we do not prepare and insert tracks,
		// which are marked for removal, because the relation between parent and
		// this track should not exists at the database
		if(!ignoreRemoved || !track->markForRemovel()){
			if(!track->prepareTrack()){
				emit removeStatusWidget(progBar);
				return false;
			}
		}
		if(track->markForRemovel()){
			// Tracks which are not in database couldn't be removed from this CD,
			// so only remove tracks, if the were in database before this store
			if(!ignoreRemoved){
				// remove the relation between this cd and the track, but don't remove the track from database!!
				query.prepare("SELECT * FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
				// there is now entry at the db, so we have nothing to do
				if(!query.first()){
					progBar->setValue(progBar->value() + 1);
					continue;
				}
				
				// remove all entries from cds_ondisc table
				unsigned int materialhasmaterialsId = query.record().value("id").toUInt();
				query.prepare(QString("DELETE FROM %1_ondisc WHERE materialhasmaterialsId=:materialhasmaterialsId")
											.arg(CDs::group()));
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
				
				query.prepare("DELETE FROM materialhasmaterials WHERE id=:id");
				query.bindValue(":id", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
		}
		else{
			// check wether relation already exists
			query.prepare("SELECT * FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
			query.bindValue(":materialsId", parent->id());
			query.bindValue(":hasMaterialsId", track->id());
			if(!query.exec()){
				emit removeStatusWidget(progBar);
				return false;
			}

			// if query first is ok, there is already an entry and we can continue to disc and trac number
			unsigned int materialhasmaterialsId = 0;
			if(!query.first()){
				// first insert relation into the materials table
				query.prepare("INSERT INTO materialhasmaterials (materialsId, hasMaterialsId) VALUES(:materialsId, :hasMaterialsId)");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}

				query.prepare("SELECT id FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				query.exec();
				if(!query.first()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
			materialhasmaterialsId = query.record().value("id").toUInt();
			
			// now enter the disc number and trac number at our own table
			if(materialhasmaterialsId < 1){
				emit removeStatusWidget(progBar);
				return false;
			}
			
			query.prepare(QString("SELECT * FROM %1_ondisc WHERE materialhasmaterialsId=:materialhasmaterialsId")
										.arg(CDs::group()));
			query.bindValue(":materialhasmaterialsId", query.record().value("id").toUInt());
			if(!query.exec()){
				emit removeStatusWidget(progBar);
				return false;
			}
			if(query.first()){
				if((track->disc() == query.record().value("disc").toUInt()) && (track->trackNumber() == query.record().value("number").toUInt())){
					progBar->setValue(progBar->value() + 1);
					continue;
				}
				query.prepare(QString("UPDATE %1_ondisc SET disc=:disc, number=:number WHERE materialhasmaterialsId=:materialhasmaterialsId").arg(CDs::group()));
				query.bindValue(":disc", track->disc());
				query.bindValue(":number", track->trackNumber());
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
			else{
				query.prepare(QString("INSERT INTO %1_ondisc (materialhasmaterialsId, disc, number) VALUES(:materialhasmaterialsId, :disc, :number)").arg(CDs::group()));
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				query.bindValue(":disc", track->disc());
				query.bindValue(":number", track->trackNumber());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
		}
		progBar->setValue(progBar->value() + 1);
	}
	
	// remove removed tracks from model
	// it must be done afterwards, because otherwise the count at the for loop isn't correct
	foreach(Medium *med, media){
		Track *track = qobject_cast<Track*>(med);
		if(!track){
			emit removeStatusWidget(progBar);
			return false;
		}
		if(track->markForRemovel()){
			removeMedium(track);
		}
	}
	
	emit removeStatusWidget(progBar);
	return true;
}

bool TracksModel::storeCollectionMode(Medium* parent, bool ignoreRemoved)
{
	if(rowCount() < 1)
		return true;
	
	if(parent->id() < 1)
		return false;
	
	MSqlDatabase db = MSqlDatabase::database();
	if( !db.isOpen() )
		return false;
	
	QProgressBar *progBar = new QProgressBar();
	progBar->setRange(0, rowCount());
	emit showStatusWidget(progBar);
	
	QSqlQuery query;
	foreach(Medium *med, media){
		Track *track = qobject_cast<Track*>(med);
		if(!track){
			emit removeStatusWidget(progBar);
			return false;
		}
		// insert our own genre
		if(track->genre() == 0){
			track->setGenre(parent->genre(), false);
			track->setGenre(parent->genreName(), false);
		}
		// insert own manufacturer
		if(track->manufacturerName().isEmpty()){
			track->setManufacturer(parent->manufacturer(), false);
			track->setManufacturer(parent->manufacturerName(), false);
		}
		// if ignoreRemoved is set to true, we do not prepare and insert tracks,
		// which are marked for removal, because the relation between parent and
		// this track should not exists at the database
		if(!ignoreRemoved || !track->markForRemovel()){
			if(!track->prepareTrack()){
				emit removeStatusWidget(progBar);
				return false;
			}
		}
		if(track->markForRemovel()){
			// Tracks which are not in database couldn't be removed from this CD,
			// so only remove tracks, if the were in database before this store
			if(!ignoreRemoved){
				// remove the relation between this cd and the track, but don't remove the track from database!!
				query.prepare("SELECT * FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
				// there is now entry at the db, so we have nothing to do
				if(!query.first()){
					progBar->setValue(progBar->value() + 1);
					continue;
				}
				
				// remove all entries from cds_ondisc table
				unsigned int materialhasmaterialsId = query.record().value("id").toUInt();
				query.prepare(QString("DELETE FROM %1_trackhastracks WHERE materialhasmaterialsId=:materialhasmaterialsId")
											.arg(CDs::group()));
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
				
				query.prepare("DELETE FROM materialhasmaterials WHERE id=:id");
				query.bindValue(":id", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
		}
		else{
			// check wether relation already exists
			query.prepare("SELECT * FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
			query.bindValue(":materialsId", parent->id());
			query.bindValue(":hasMaterialsId", track->id());
			if(!query.exec()){
				emit removeStatusWidget(progBar);
				return false;
			}

			// if query first is ok, there is already an entry and we can continue to disc and trac number
			unsigned int materialhasmaterialsId = 0;
			if(!query.first()){
				// first insert relation into the materials table
				query.prepare("INSERT INTO materialhasmaterials (materialsId, hasMaterialsId) VALUES(:materialsId, :hasMaterialsId)");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}

				query.prepare("SELECT id FROM materialhasmaterials WHERE materialsId=:materialsId AND hasMaterialsId=:hasMaterialsId");
				query.bindValue(":materialsId", parent->id());
				query.bindValue(":hasMaterialsId", track->id());
				query.exec();
				if(!query.first()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
			materialhasmaterialsId = query.record().value("id").toUInt();
			
			// now enter the disc number and trac number at our own table
			if(materialhasmaterialsId < 1){
				emit removeStatusWidget(progBar);
				return false;
			}
			
			query.prepare(QString("SELECT * FROM %1_trackhastracks WHERE materialhasmaterialsId=:materialhasmaterialsId")
										.arg(CDs::group()));
			query.bindValue(":materialhasmaterialsId", query.record().value("id").toUInt());
			if(!query.exec()){
				emit removeStatusWidget(progBar);
				return false;
			}
			if(query.first()){
				if((track->disc() == query.record().value("disc").toUInt()) && (track->trackNumber() == query.record().value("number").toUInt())){
					progBar->setValue(progBar->value() + 1);
					continue;
				}
				query.prepare(QString("UPDATE %1_trackhastracks SET number=:number WHERE materialhasmaterialsId=:materialhasmaterialsId").arg(CDs::group()));
				query.bindValue(":number", track->trackNumber());
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
			else{
				query.prepare(QString("INSERT INTO %1_trackhastracks (materialhasmaterialsId, number) VALUES(:materialhasmaterialsId, :number)").arg(CDs::group()));
				query.bindValue(":materialhasmaterialsId", materialhasmaterialsId);
				query.bindValue(":number", track->trackNumber());
				if(!query.exec()){
					emit removeStatusWidget(progBar);
					return false;
				}
			}
		}
		progBar->setValue(progBar->value() + 1);
	}
	
	// remove removed tracks from model
	// it must be done afterwards, because otherwise the count at the for loop isn't correct
	foreach(Medium *med, media){
		Track *track = qobject_cast<Track*>(med);
		if(!track){
			emit removeStatusWidget(progBar);
			return false;
		}
		if(track->markForRemovel()){
			removeMedium(track);
		}
	}
	
	emit removeStatusWidget(progBar);
	return true;
}

#include "tracksmodel.moc"
