/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "mediendb.h"

#include "setupwizard.h"
#include "modulelistwidgetitem.h"
#include "mediaplugin.h"
#include "searchplugin.h"
#include "modulestatedialog.h"
#include "searchwidget.h"
#include "settingsdialog.h"
#include "editmanufacturers.h"
#include "editgenres.h"
#include "editmaterialpersons.h"
#include "editroles.h"
#include "edituser.h"
#include "shoppingcart.h"
#include "summary.h"
#include "projects.h"
#include "about.h"
#include "helpcenter.h"
#include "dependencies.h"
#include <dbbackupdialog.h>
#include "editmintypes.h"

#include "updatenoticedialog.h"

#include <QSettings>
#include <QMessageBox>
#include <QDir>
#include <QPluginLoader>
#include <QSqlDatabase>
#include <QDockWidget>
#include <QToolBar>
#include <QTextStream>

#include <stdio.h>

MedienDB::MedienDB ( int& argc, char** argv, QWidget* parent, Qt::WindowFlags flags ) : QMainWindow ( parent, flags )
{
	shoppingCart = 0;
	helpCenter = 0;
	doExit = false;
	
	processCommandlineArgs(argc, argv);
	
  setupUi ( this );

  QSettings *settings = new QSettings();
  shortName = settings->value ( "appNameShort" ).toString();
  QString appTitle = settings->value ( "appName" ).toString();
  setWindowTitle ( appTitle );
	setWindowIcon(QIcon(":icons/128x128/meganizer.png"));
	
#if QT_VERSION_MINOR < 6
	fileExitAction->setIcon(QIcon(":/icons/exit.png"));
	editSettingsAction->setIcon(QIcon(":/icons/settings.png"));
	helpHelpAction->setIcon(QIcon(":/icons/help.png"));
// 	helpAboutAction->setIcon(QIcon());
#else
	fileExitAction->setIcon(QIcon::fromTheme("application-exit",
																					 QIcon(":/icons/exit.png")));
	editSettingsAction->setIcon(QIcon::fromTheme("package-settings",
																								QIcon(":/icons/settings.png")));
	helpHelpAction->setIcon(QIcon::fromTheme("help-contents",
																					 QIcon(":/icons/help.png")));
	helpAboutAction->setIcon(QIcon::fromTheme("help-about"));
#endif
	fileUserAction->setIcon(QIcon(":icons/edituser.png"));

	connect ( fileProjectsAction, SIGNAL ( triggered()), this, SLOT(fileProjects()));
	connect ( fileGenresAction, SIGNAL ( triggered() ), this, SLOT( fileGenres()));
	connect ( fileManufacturersAction, SIGNAL ( triggered() ), this, SLOT( fileManufacturers()));
	connect ( fileMaterialPersonsAction, SIGNAL( triggered() ), this, SLOT(fileMaterialPersons()));
	connect ( fileRolesAction, SIGNAL ( triggered() ), this, SLOT(fileRoles()));
	connect ( fileUserAction, SIGNAL ( triggered() ), this, SLOT(fileUser()));
	connect ( fileMinTypesAction, SIGNAL ( triggered() ), this, SLOT(fileMinTypes()));
  connect ( fileExitAction, SIGNAL ( triggered() ), this, SLOT ( fileExit() ) );
	connect ( editSetupWizardAction, SIGNAL( triggered() ), this, SLOT ( editSetupWizard() ));
	connect ( editDBBackupAction, SIGNAL( triggered() ), this, SLOT ( editDBBackup()));
	connect ( editSettingsAction, SIGNAL ( triggered() ), this, SLOT ( editSettings() ) );
	connect ( viewSummaryAction, SIGNAL ( triggered() ), this, SLOT ( viewSummary() ) );
  connect ( helpHelpAction, SIGNAL ( triggered() ), this, SLOT ( helpHelp() ) );
  connect ( helpUsedSoftwareAction, SIGNAL ( triggered() ), this, SLOT ( helpUsedSoftware() ) );
  connect ( helpAboutPluginsAction, SIGNAL ( triggered() ), this, SLOT ( helpAboutPlugins() ) );
	connect ( helpAboutAction, SIGNAL ( triggered() ), this, SLOT ( helpMedienDB() ) );
}

MedienDB::~MedienDB()
{
	close();
	// we must unload all widgets, 
	// to remove all remaining database objects
	unloadPlugins();
	setCentralWidget(0);
	foreach(QPointer<QDockWidget> dock, dockWidgets){
		if(dock)
			removeDockWidget(dock);
			delete dock;
	}
	QStringList connNames = QSqlDatabase::connectionNames();
  foreach ( QString name, connNames ) {
    QSqlDatabase::removeDatabase ( name );
  }
}

bool MedienDB::shouldExit()
{
	return doExit;
}

void MedienDB::init()
{
  loadPlugins();

  showSetupWizard();
	connect ( this, SIGNAL ( checkSetupWizard() ), this, SLOT ( showSetupWizard() ) );

	shoppingCart = new ShoppingCart(searchPlugins, this);
	connectMSlots(shoppingCart);
	shoppingCart->init(true);
	
  // display search widget
  SearchWidget *searchWidget = new SearchWidget ( this );
  connectMSlots ( searchWidget );
  searchWidget->init ( mediaPlugins, searchPlugins, shoppingCart );
	
	// starting with summary
	viewSummary();
	
	QSettings settings;
	settings.beginGroup("geometries");
  restoreGeometry(settings.value(objectName() +"_geometry").toByteArray());
  restoreState(settings.value(objectName() +"_state").toByteArray());
	settings.endGroup();
	
	// check if update is available
	UpdateNoticeDialog *updateNotice = new UpdateNoticeDialog(this);
	connectMSlots(updateNotice);
	updateNotice->check();
}

void MedienDB::closeEvent(QCloseEvent* event)
{
	QSettings settings;
	settings.beginGroup("geometries");
  settings.setValue(objectName() +"_geometry", saveGeometry());
  settings.setValue(objectName() +"_state", saveState());
	settings.endGroup();
	
	if(helpCenter){
		delete helpCenter;
		helpCenter = 0;
	}
	
  QMainWindow::closeEvent(event);
}

bool MedienDB::loadPlugins()
{
  // result will be false if less than one media plugin is found
  // a search plugin isn't needed, because a local database is thinkable
  bool result = false;
  QSettings settings;
  QString pluginDir = settings.value ( "pluginDir" ).toString();

  if ( pluginDir.isEmpty() ) {
    pluginDir = QString ( PLUGINDIR );
  }

  //load and test plugin
  QDir dir ( pluginDir );

  if ( !dir.exists() ) {
    error ( tr ( "Plugin dir does not exist!\n%1" ).arg ( pluginDir ) );
    return false;
  }

  QStringList files = dir.entryList ( QDir::Files );

  if ( files.size() < 1 ) {
    error ( tr ( "No files found at \n%1" ).arg ( pluginDir ) );
    return false;
  }

  foreach ( QString file, files ) {

    loadPlugin ( dir.filePath ( file ) );
  }

  if ( result )
    emit checkSetupWizard();

  return result;
}

bool MedienDB::loadPlugin ( ModuleListWidgetItem* plugin, bool emitSetupWizard )
{
  return loadPlugin ( plugin->file(), emitSetupWizard );
}

bool MedienDB::loadPlugin ( QString file, bool emitSetupWizard )
{
  ModuleListWidgetItem *item = new ModuleListWidgetItem();
  pluginProtocol.append ( item );
  item->setFile ( file );
  QPluginLoader loader ( file );
  QObject *plugin = loader.instance();

  if ( !plugin ) {
    item->setStatus ( false );
    item->setErrorMessage ( tr ( "While loading %1 an error occured:\n%2 %1" ).arg ( file ).arg(loader.errorString()) );
    information ( tr("Could not load %1! Details at Help->Plugins").arg(item->name()) );
    return false;
  }

  // first try to find media plugins
  MediaPlugin *newPlugin = qobject_cast<MediaPlugin *> ( plugin );

  if ( !newPlugin ) {
    // may be it's a search plugin
    SearchPlugin *searchPlugin = qobject_cast<SearchPlugin *> ( plugin );
    if(!searchPlugin){
      item->setStatus ( false );
      item->setErrorMessage ( tr ( "Plugin is not a Meganizer-plugin:\n %1" ).arg ( file ) );
      information ( tr("Could not load %1! Details at Help->Plugins").arg(item->name()) );
      return false;
    }
    item->setName(searchPlugin->name());
		item->setModuleIcon(searchPlugin->icon());
    item->setDescription( searchPlugin->description());
    connectMSlots(searchPlugin);
    
    if ( !searchPlugin->initiate() ) {
      item->setStatus ( false );
      item->setErrorMessage ( tr ( "Cannot initiate module\n %1" ).arg ( file ) );
      information ( tr("Could not load %1! Details at Help->Plugins").arg(item->name()) );
      return false;
    }

    item->setStatus ( true );

    item->setErrorMessage ( "" );
    searchPlugins.append ( searchPlugin );


    if ( emitSetupWizard )
      emit checkSetupWizard();

    return true;
  }

  // init and load media plugin
  item->setName ( newPlugin->name() );
	item->setModuleIcon(newPlugin->icon());
  item->setDescription ( newPlugin->description() );

  connectMSlots ( newPlugin );

  if ( !newPlugin->initiate() ) {
    item->setStatus ( false );
    item->setErrorMessage ( tr ( "Cannot initiate module\n %1" ).arg ( file ) );
    information ( tr("Could not load %1! Details at Help->Plugins").arg(item->name()) );
    return false;
  }

  item->setStatus ( true );

  item->setErrorMessage ( "" );
  mediaPlugins.append ( newPlugin );


  if ( emitSetupWizard )
    emit checkSetupWizard();

  return true;
}

bool MedienDB::unloadPlugin ( ModuleListWidgetItem *item )
{
  // try media plugins
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    if ( item->name() == plugin->name() ) {
      return unloadPlugin ( plugin );
    }
  }

  foreach ( SearchPlugin *plugin, searchPlugins ) {
    if ( item->name() == plugin->name() ) {
      return unloadPlugin ( plugin );
    }
  }
  return false;
}

bool MedienDB::unloadPlugin ( QObject *plugin )
{
  MediaPlugin *object = qobject_cast<MediaPlugin*> ( plugin );

  if ( object ) {
    object->unload();
    mediaPlugins.removeAll ( object );
    foreach ( ModuleListWidgetItem *item, pluginProtocol ) {
      if ( item->name() == object->name() ) {
        pluginProtocol.removeAll ( item );
        delete item;
        break;
      }
    }

    delete object;
    return true;
  }

  SearchPlugin *search = qobject_cast<SearchPlugin*> ( plugin );

  if ( search ) {
    search->unload();
    searchPlugins.removeAll ( search );
    foreach ( ModuleListWidgetItem *item, pluginProtocol ) {
      if ( item->name() == search->name() ) {
        pluginProtocol.removeAll ( item );
        delete item;
        break;
      }
    }

    delete search;
    return true;
  }

  return false;
}

bool MedienDB::unloadPlugins()
{
  bool result = true;
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    if ( !unloadPlugin ( plugin ) )
      result = false;
  }

  foreach ( SearchPlugin *plugin, searchPlugins ) {
    if ( !unloadPlugin ( plugin ) )
      result = false;
  }

  return result;
}

bool MedienDB::reloadPlugins()
{
  if ( !unloadPlugins() )
    return false;

  if ( !loadPlugins() )
    return false;

  return true;
}

void MedienDB::showSetupWizard(bool msgOnNotNeeded)
{
  // check for needed updates or other things which needs to display the setupwizard
  SetupWizard setupWizard ( mediaPlugins, searchPlugins,
														username, host, database, driver, password,
														this );
  connectMSlots(&setupWizard);

  if ( setupWizard.isNeeded() ){
    setupWizard.exec();
	}
	else{
		if(msgOnNotNeeded)
			QMessageBox::information(this, tr("Meganizer - SetupWizard"), tr("Everything setted up already"));
	}
	
// 	setupWizard.deleteLater();
}

void MedienDB::fileProjects()
{
	Projects *projects = new Projects(this);
	connectMSlots(projects);
	setCentralWidget(projects);
}

void MedienDB::fileGenres()
{
	EditGenres *editGenre = new EditGenres(this);
	connectMSlots(editGenre);
	editGenre->setEditMode();
	editGenre->exec();
	delete editGenre;
}

void MedienDB::fileManufacturers()
{
	EditManufacturers *editManu = new EditManufacturers(this);
	editManu->setEditMode();
	connectMSlots(editManu);
	editManu->exec();
	delete editManu;
}

void MedienDB::fileMaterialPersons()
{
	EditMaterialPersons *editMatPer = new EditMaterialPersons(this);
	connectMSlots(editMatPer);
	editMatPer->setEditMode();
	editMatPer->exec();
	delete editMatPer;
}

void MedienDB::fileRoles()
{
	EditRoles *editRoles = new EditRoles(this);
	connectMSlots(editRoles);
	editRoles->setEditMode();
	editRoles->exec();
	delete editRoles;
}

void MedienDB::fileUser()
{
	EditUser *editUser = new EditUser(this);
	connectMSlots(editUser);
	editUser->setEditMode();
	editUser->exec();
	delete editUser;
}

void MedienDB::fileMinTypes()
{
	EditMinTypes editMinType(this);
	connectMSlots(&editMinType);
	editMinType.setEditMode();
	editMinType.exec();
	editMinType.disconnect(this);
	disconnect(&editMinType);
}

void MedienDB::fileExit()
{
  close();
}

void MedienDB::editSetupWizard()
{
	showSetupWizard(true);
}

void MedienDB::editDBBackup()
{
	DBBackupDialog *dia = new DBBackupDialog(this);
	connectMSlots(dia);
	dia->exec();
	delete dia;
}

void MedienDB::editSettings()
{
	SettingsDialog settingsDia(mediaPlugins, searchPlugins, this);
	connectMSlots(&settingsDia);
	settingsDia.exec();
}

void MedienDB::viewSummary()
{
	Summary *summary = new Summary(this);
	connectMSlots(summary);
	setCentralWidget(summary);
}

void MedienDB::helpHelp()
{
	helpHelp(QString());
}

void MedienDB::helpHelp ( QString file )
{
  if(!helpCenter){
		helpCenter = new HelpCenter(this);
		connectMSlots(helpCenter);
	}
	if(file.isEmpty()){
		helpCenter->showDocumentation();
	}
	else{
		helpCenter->showDocumentation(file);
	}
}

void MedienDB::helpMedienDB()
{
  About *about = new About(this);
	about->exec();
	delete about;
}

void MedienDB::helpUsedSoftware()
{
  Dependencies *dep = new Dependencies(this);
	connectMSlots(dep);
	dep->init();
	dep->exec();
	delete dep;
}

void MedienDB::helpAboutPlugins()
{
  ModuleStateDialog *moduleState = new ModuleStateDialog ( pluginProtocol, this );
  connectMSlots ( moduleState );
  moduleState->exec();
	delete moduleState;
}

void MedienDB::error ( QString msg )
{
  QMessageBox::critical ( this, shortName, msg );
}

void MedienDB::warning ( QString msg )
{
  QMessageBox::warning ( this, shortName, msg );
}

void MedienDB::information ( QString msg )
{
  statusbar->showMessage ( msg );
}

void MedienDB::addNewDockWidget ( QDockWidget* widget, Qt::DockWidgetArea area )
{
  addDockWidget ( area, widget );
	dockWidgets << widget;
}

void MedienDB::newCentralWidget ( QWidget *widget )
{
  setCentralWidget ( widget );
}

void MedienDB::newMenu ( QMenu *newMenu, QString parent )
{
  if ( parent.isEmpty() ) {
    menubar->addMenu ( newMenu );
    return;
  }

  QMenu *parentMenu = menubar->findChild<QMenu *> ( parent );

  if ( parentMenu ) {
    parentMenu->addMenu ( newMenu );
    return;
  }

  menubar->addMenu ( newMenu );
}

void MedienDB::newAction ( QAction *newAction, QString parent )
{
  if ( parent.isEmpty() ) {
    menubar->addAction ( newAction );
    return;
  }

  // try to add it to a main menu
  QMenu *parentMenu = menubar->findChild<QMenu *> ( parent );

  if ( parentMenu ) {
    parentMenu->addAction ( newAction );
    return;
  }

  // if it is not a main menu, try all child menus
  parentMenu = 0;

  QList<QMenu *> menus = menubar->findChildren<QMenu *>();

  for ( int i = 0; i < menus.size(); i++ ) {
    parentMenu = menus[i]->findChild<QMenu *> ( parent );

    if ( parentMenu ) {
      parentMenu->addAction ( newAction );
      return;
    }
  }

  // menu doesn't exist, so we don't know what to do
}

void MedienDB::newToolbar(QToolBar* newBar)
{
	addToolBar(newBar);
}

void MedienDB::delToolbar(QToolBar* delBar)
{
	removeToolBar(delBar);
}

void MedienDB::showStatusWidget(QWidget *widget){
	statusbar->addPermanentWidget(widget);
//   statusbar->insertPermanentWidget(-1, widget);
}

void MedienDB::removeStatusWidget(QWidget *widget){
  statusbar->removeWidget(widget);
	widget->deleteLater();
}
void MedienDB::connectMSlots ( QObject *object )
{
  connect ( object, SIGNAL ( error ( QString ) ), this, SLOT ( error ( QString ) ) );
  connect ( object, SIGNAL ( warning ( QString ) ), this, SLOT ( warning ( QString ) ) );
  connect ( object, SIGNAL ( information ( QString ) ), this, SLOT ( information ( QString ) ) );

  connect ( object, SIGNAL ( addNewDockWidget ( QDockWidget*, Qt::DockWidgetArea ) ),
            this, SLOT ( addNewDockWidget ( QDockWidget*, Qt::DockWidgetArea ) ) );
  connect ( object, SIGNAL ( setCentralWidget ( QWidget* ) ), this, SLOT ( newCentralWidget ( QWidget* ) ) );
  connect ( object, SIGNAL ( addMenu ( QMenu*, QString ) ), this, SLOT ( newMenu ( QMenu*, QString ) ) );
  connect ( object, SIGNAL ( addAction ( QAction*, QString ) ), this, SLOT ( newAction ( QAction*, QString ) ) );
	connect(object, SIGNAL(addToolbar(QToolBar*)), this, SLOT(newToolbar(QToolBar*)));
  connect(object, SIGNAL(delToolbar(QToolBar*)), this, SLOT(delToolbar(QToolBar*)));
	
	connect(object, SIGNAL(showStatusWidget(QWidget*)), this, SLOT(showStatusWidget(QWidget*)));
  connect(object, SIGNAL(removeStatusWidget(QWidget*)), this, SLOT(removeStatusWidget(QWidget*)));
	
	connect(object, SIGNAL(showHelp(QString)), this, SLOT(helpHelp(QString)));
}

void MedienDB::processCommandlineArgs(int& argc, char** argv)
{
	// processing command line arguments
	for(int i = 0; i < argc; i++){
		if(QString(argv[i]) == "-u"){
			if(i + 1 >= argc){
				showHelp();
			}
			else{
				username = QString(argv[i + 1]);
			}
		}
		if(QString(argv[i]) == "-p"){
			if(i + 1 >= argc){
				showHelp();
			}
			else{
				password = QString(argv[i + 1]);
			}
		}
		if(QString(argv[i]) == "-s"){
			if(i + 1 >= argc){
				showHelp();
			}
			else{
				host = QString(argv[i + 1]);
			}
		}
		if(QString(argv[i]) == "-d"){
			if(i + 1 >= argc){
				showHelp();
			}
			else{
				database = QString(argv[i + 1]);
			}
		}
		if(QString(argv[i]) == "-t"){
			if(i + 1 >= argc){
				showHelp();
			}
			else{
				driver = QString(argv[i + 1]);
			}
		}
		if(QString(argv[i]) == "-v"){
			QSettings settings;
			QTextStream out(stdout);
			out << tr("Version: ") + settings.value("appVersion").toString() << endl;
			doExit = true;
		}
		if((QString(argv[i]) == "-h") || (QString(argv[i]) == "--help")){
			showHelp();
		}
	}
}

void MedienDB::showHelp()
{
	QSettings settings;
	QTextStream out(stdout);
	out << settings.value("appNameShort").toString() << settings.value("appVersion").toString() << endl;
	out << tr("Usage: meganizer [-u name] [-p password] [-s db-host] [-d db-name] [-t driver]") << endl;
	out << endl;
	out << tr("For details see manpage \"meganizer\" (Execute man meganizer)");
	out << endl;
	out << QString(" -u        [username]  %1").arg(tr("username to connect to the database")) << endl;
	out << QString(" -p        [password]  %1").arg(tr("password to connect to the database")) << endl;
	out << QString(" -s        [host]      %1").arg(tr("host where the database is running on")) << endl;
	out << QString(" -d        [name]      %1").arg(tr("name of the database which should be used")) << endl;
	out << QString(" -t        [driver]    %1").arg(tr("Driver to connect to database (Should be QMYSQL)")) << endl;
	out << QString(" -v                    %1").arg(tr("Show the version and exit")) << endl;
	out << QString(" -h --help             %1").arg(tr("Show this help and exit")) << endl;
	doExit = true;
}

#include "mediendb.moc"
