/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "setupwizard.h"

#include "setupwizardwidget.h"
#include "setupwizarddb.h"
#include "setupuser.h"
#include "sqllogin.h"
#include "dbinit.h"
#include "mediaplugin.h"
#include "searchplugin.h"
#include "setupwizarddbdump.h"
#include "dbupdatewidget.h"
#include "sqllogin.h"

#include <QMessageBox>
#include <QSettings>

SetupWizard::SetupWizard ( QList<MediaPlugin*> newMediaPlugins,
													 QList<SearchPlugin*> newSearchPlugins,
													 QString username, QString host,
													 QString database, QString driver,
													 QString password,
													 QWidget *parent, Qt::WindowFlags fl )
 : MDialog ( parent, fl )
{
  needed = true;
	doReload = false;
  mediaPlugins = newMediaPlugins;
  searchPlugins = newSearchPlugins;

  setupUi ( this );
	setHelp("keyword:Setupwizard", helpButton);
	
  QSettings *settings = new QSettings();
  shortName = settings->value ( "appNameShort" ).toString();
  setWindowTitle ( shortName + " " + windowTitle() );

  firstTitle = titleLabel->text();

  connect ( stackedWidget, SIGNAL ( currentChanged ( int ) ), this, SLOT ( setProgress() ) );
  connect ( stackedWidget, SIGNAL ( widgetRemoved ( int ) ), this, SLOT ( setProgress() ) );

  connect ( cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
  connect ( nextButton, SIGNAL ( clicked() ), this, SLOT ( next() ) );
  connect ( backButton, SIGNAL ( clicked() ), this, SLOT ( back() ) );

	this->username = username;
	this->host = host;
	this->database = database;
	this->driver = driver;
	this->password = password;
	
  loadObjects();
}

SetupWizard::~SetupWizard()
{
  close();
}

bool SetupWizard::isNeeded()
{
  return needed;
}

void SetupWizard::reload()
{
	doReload = false;
	unloadObjects();
	loadObjects();
	stackedWidget->setCurrentIndex(1);
}

int SetupWizard::exec()
{
  if ( isNeeded() )
    return QDialog::exec();

  return QDialog::Accepted;
}

void SetupWizard::closeEvent ( QCloseEvent *event )
{
}

void SetupWizard::setProgress()
{
  int index = stackedWidget->currentIndex();
  progressBar->setRange ( 0, stackedWidget->count() );
  progressBar->setValue ( index );

  // disable and enable back and next button

  if ( index == 0 ) {
    backButton->setEnabled ( false );
    titleLabel->setText ( firstTitle );
  }

  else {
    backButton->setEnabled ( true );
    SetupWizardWidget *widget = qobject_cast<SetupWizardWidget*> ( stackedWidget->currentWidget() );

    if ( widget )
      titleLabel->setText ( widget->name() );
  }

  if ( index >=  stackedWidget->count() - 1 ) {
    nextButton->setText ( tr ( "&Finish" ) );
  }

  else {
    nextButton->setText ( tr ( "&Next" ) );
  }
}

void SetupWizard::next()
{
	int index = stackedWidget->currentIndex();
	if(index != 0){
		// the first widget is no setupwizardwidget
	  // save settings of last widget
		SetupWizardWidget *widget = qobject_cast<SetupWizardWidget*> ( stackedWidget->currentWidget() );
		if ( !widget ){
			showErrorMessage ( tr ( "Could not save settings" ) );
			return;
		}
		
		if ( !widget->wizardObject() ){
			showErrorMessage ( tr ( "Could not save settings" ) );
			return;
		}
			
		if ( !widget->wizardObject()->finish ( widget ) ) {
			// only emit error if we don't have to reload,
			// because the function don't return something, because the objects 
			// are deleted before they can return a value
			showErrorMessage ( tr ( "Could not save settings" ) );
			return;
		}
		if(doReload){
			reload();
			return;
		}
	}  

  if ( index >=  stackedWidget->count() - 1 ) {
    // finish setup
    accept();
  }

  else {
    stackedWidget->setCurrentIndex ( index + 1 );
  }
}

void SetupWizard::back()
{
  if ( stackedWidget->currentIndex() > 0 )
    stackedWidget->setCurrentIndex ( stackedWidget->currentIndex() - 1 );
}

void SetupWizard::markReload()
{
	doReload = true;
}

void SetupWizard::showErrorMessage ( QString msg )
{
  QMessageBox::critical ( this, shortName, msg );
}

void SetupWizard::loadObjects()
{
  // check database
	DBInit *dbInit = new DBInit(this, username, host, database, driver, password);
  SetupWizardDB *dbWizard = new SetupWizardDB ( tr ( "Configure and Update Database" ), dbInit, this );
  connect ( dbWizard, SIGNAL ( error ( QString ) ), this, SLOT ( showErrorMessage ( QString ) ) );
  connect ( dbWizard, SIGNAL ( warning ( QString ) ), this, SLOT ( showErrorMessage ( QString ) ) );
	addObjectsWidgets ( dbWizard );

	// check actual user
  SetupUser *setupUser = new SetupUser ( this );
  connect ( setupUser, SIGNAL ( error ( QString ) ), this, SLOT ( showErrorMessage ( QString ) ) );
  connect ( setupUser, SIGNAL ( warning ( QString ) ), this, SLOT ( showErrorMessage ( QString ) ) );
  addObjectsWidgets ( setupUser );

	// check MediaPlugins
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    SetupWizardObject *object = plugin->wizardObject();

    if ( !object )
      continue;

    addObjectsWidgets ( object );
  }

  // check SearchPlugins
  foreach ( SearchPlugin *plugin, searchPlugins ) {
    SetupWizardObject *object = plugin->wizardObject();

    if ( !object )
      continue;

    addObjectsWidgets ( object );
  }

	// if we have not added any SetupWizardWidgets's, a setup is not needed
  if ( stackedWidget->count() == 1 ){
    needed = false;
		accept();
	}

	// if one SetupWizardDB is present, insert DBDumpObject as first object
	QList<SetupWizardWidget*> widgets;
	bool insertBackup = false;
	for(int i = 0; i < stackedWidget->count(); i++){
		SetupWizardWidget *widget = qobject_cast<SetupWizardWidget*>(stackedWidget->widget(i));
		if(!widget)
			continue;
		
		widgets << widget;
		
		if(!insertBackup){
			DBUpdateWidget *dbWidget = qobject_cast<DBUpdateWidget*>(stackedWidget->widget(i));
			if(dbWidget)
				insertBackup = true;
		}
	}
	if(insertBackup){
		SetupWizardDBDump *dbDump = new SetupWizardDBDump(this);
		if(dbDump->needed()){
			// the insert widget should be the first one
			foreach(SetupWizardWidget *widget, widgets)
				stackedWidget->removeWidget(widget);
			
			addObjectsWidgets(dbDump);
			foreach(SetupWizardWidget *widget, widgets)
				stackedWidget->addWidget(widget);
		}
	}

  // at last set the progress to start
  setProgress();
}

void SetupWizard::addObjectsWidgets ( SetupWizardObject *object )
{
	object->setParent(this);
  if ( !object )
    return;

  if ( !object->needed() )
    return;

  QList<SetupWizardWidget*> widgets = object->wizardWidgets();

  if ( widgets.size() == 0 ) {
    emit error ( tr ( "Could not get needed Widget! %1" ).arg ( object->title() ) );
    return;
  }

  foreach ( MWidget *widget, widgets ) {
    connect ( widget, SIGNAL ( error ( QString ) ), this, SLOT ( showErrorMessage ( QString ) ) );
		connect ( widget, SIGNAL ( reloadSettings()), this, SLOT( markReload()));
    stackedWidget->addWidget ( widget );
  }
}

void SetupWizard::unloadObjects()
{
	stackedWidget->setCurrentIndex(0);
	
	QList<SetupWizardObject *> objects;
	QList<SetupWizardWidget*> widgets;
	for(int i = 0; i < stackedWidget->count(); i++){
		SetupWizardWidget *widget = qobject_cast<SetupWizardWidget*>(stackedWidget->widget(i));
		if(!widget)
			continue;
		widgets.append(widget);

		SetupWizardObject *object = widget->wizardObject();
		if(!object)
			continue;
		if(objects.indexOf(object) == -1)
			objects.append(object);
	}
	// we must delete objects at last, because more than one widget can reference one object
	qDeleteAll(widgets);
	qDeleteAll(objects);
}

#include "setupwizard.moc"
