/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "statisticssummary.h"

#include <msqldatabase.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QPainter>

#include <cmath>

StatisticsSummary::StatisticsSummary ( QWidget* parent, Qt::WindowFlags flags ) : MWidget ( parent, flags )
{
	setObjectName("statisticsSummaryWidget");
	setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	setMinimumSize(200, 200);
	
	unsigned int myUserId = 0;
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.prepare("SELECT * FROM users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUserId = query.record().value("id").toUInt();
		
		// build mapping for our media types
		query.exec("SELECT COUNT(mats.id), types.name "
							 "  FROM viewmaterials mats, materialtypes types"
							 "  WHERE mats.usersId=1 AND mats.materialtypesId=types.id"
							 "  GROUP BY materialtypesId");
		while(query.next()){
			mapTypeCount.insert
			(
				query.record().value("name").toString(),
				query.record().value("COUNT(mats.id)").toInt()
			);
		}
		
		// build user medias
		query.exec("SELECT COUNT(mats.id), users.name, users.givenName"
							 "  FROM viewmaterials mats, users"
							 "  WHERE mats.usersId=users.id"
							 "  GROUP BY mats.usersId");
		while(query.next()){
			QString name = query.record().value("name").toString();
			QString givenName = query.record().value("givenName").toString();
			if(!givenName.isEmpty())
				name += ", "+ givenName.left(1);
			mapUserCount.insert
			(
				name,
				query.record().value("COUNT(mats.id)").toInt()
			);
		}
		
		if(closeDB)
			db.close();
	}
	
	PI = acos(-1);
	margin = 4;
	
	colorNames << "darkRed" << "darkGreen" << "darkBlue";
	colorNames << "red" << "green" << "blue" << "yellow" << "black";
	colorNames << "darkCyan" << "darkMagenta" <<  "darkGrey";
	colorNames << "cyan" << "magenta";
}

void StatisticsSummary::paintEvent ( QPaintEvent* event )
{
	int height = int(float(size().height() - 3 * margin)/2);
	int width = size().width() - 2 * margin;
	
	QRect typeChart, typePieRect, userChart, userPieRect;
	typeChart.setLeft(margin);
	typeChart.setTop(margin);
	typeChart.setWidth(width);
	typeChart.setHeight(height);
	
	userChart.setLeft(typeChart.left());
	userChart.setTop(typeChart.bottom() + 2 * margin);
	userChart.setWidth(width);
	userChart.setHeight(height);
	
	// draw type chart
	drawPie(typeChart, mapTypeCount);
	
	// draw user chart
	drawPie(userChart, mapUserCount);
	
  QWidget::paintEvent ( event );
}

void StatisticsSummary::drawPie(QRect chartRect, QMap<QString, int> quantities)
{
	int square = chartRect.width() - 3 * margin;
	// make them a square
	if(square > chartRect.height() - 2 * margin)
		square = chartRect.height() - 2 * margin;
	// setting pie rect
	QRect pieRect;
	pieRect.setLeft(chartRect.left() + margin);
	pieRect.setTop(chartRect.top() + margin);
	pieRect.setWidth(square);
	pieRect.setHeight(square);
	
	QPainter painter(this);
	double radius = float(pieRect.width()) / 2.0; 
	double area = PI*pow(radius, 2);
	QPoint origin;
	origin.setY(pieRect.y() + radius);
	origin.setX(pieRect.x() + radius);
	
	float total = 0;
	QString textLength;
	foreach(QString key, quantities.uniqueKeys()){
		total += quantities.value(key);
		QString check = key +" - 100%";
		if(textLength.length() < check.length())
			textLength = check;
	}
	
	// calc angles
	QList<QList<int> > angles;
	int startAngle = 0;
	foreach(float count, quantities){
		double piece = area * count / total;
		double spanAngle = 16.0 * piece * 360.0 / PI / pow(radius, 2);
		QList<int> values;
		values << startAngle;
		values << spanAngle;
		angles.append(values);
		startAngle += spanAngle;
	}
	
	// drawing background white
	chartRect.setWidth(pieRect.width() + 2 * margin + painter.fontMetrics().width(textLength));
	painter.fillRect(chartRect, QBrush(QColor(255,255,255)));
	
	// draw pie
	QColor brushColor;
	for(int i = 0; i < angles.size(); i++){
		if(i <= colorNames.size())
			brushColor.setNamedColor(colorNames[i]);
		painter.setBrush(QBrush(brushColor));
		painter.drawPie(pieRect, angles[i][0], angles[i][1]);
	}

	// draw legend
	QPoint textPoint = origin;
	textPoint.setX(pieRect.right() + 2 * margin);
	textPoint.setY(chartRect.top() + margin + painter.fontMetrics().height());
	int i = 0;
	foreach(QString key, quantities.uniqueKeys()){
		if(i <= colorNames.size())
			brushColor.setNamedColor(colorNames[i]);
		
 		painter.setPen(brushColor);
		painter.drawText(textPoint, key +" - "+ QString::number(float(quantities.value(key)) / total * 100.0, 'f', 0) + "%");
		textPoint.setY(textPoint.y() + painter.fontMetrics().height() + margin);
		i++;
	}
	
	painter.drawText(textPoint.x(), chartRect.bottom() - margin, tr("Total: %1").arg(total));
}

#include "statisticssummary.moc"
