/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "meganizerconfig.h"
#include "updatenoticedialog.h"

#include "openfileexternal.h"

#include <QSettings>
#include <QNetworkReply>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QDomDocument>
#include <QDate>

UpdateNoticeDialog::UpdateNoticeDialog(QWidget* parent, Qt::WindowFlags flags)
	: MDialog(parent, flags)
{
#if defined(Q_OS_WIN32)
	platform = "win32";
# endif
#if defined(Q_OS_MACX)
	platform = "mac";
#endif
#if not defined(BUILD_PACKAGE)
	#if defined(Q_OS_LINUX)
			platform = "linux";
	#endif
#endif
	
	setupUi(this);
	connect(labelInformation, SIGNAL(linkActivated(QString)), this, SLOT(openLink(QString)));
	connect(dontShowAgainEdit, SIGNAL(stateChanged(int)), this, SLOT(dontShowAgain(int)));
	connect(closeButton, SIGNAL(clicked(bool)), this, SLOT(reject()));
	
	QSettings settings;
	headerLabel->setText(settings.value("appName").toString());
}

void UpdateNoticeDialog::check()
{
	if(platform.isEmpty())
		return;
	
	QSettings settings;
	QDate lastCheck = settings.value("updateCheck").toDate();
	if(!lastCheck.isNull() && lastCheck.daysTo(QDate::currentDate()) < 2){
		// get last available new version
		QString newVersion = settings.value("newAppVersion").toString();
		QRegExp regVer("([0-9]+)\\.([0-9]+)\\.([0-9]+)");
		if(regVer.indexIn(newVersion) == -1)
			return;
		int major = regVer.cap(1).toInt();
		int minor = regVer.cap(2).toInt();
		int patch = regVer.cap(3).toInt();
		int rc = 0;
		QString rcString;
		QRegExp regRc("-rc([0-9]+)");
		if(regRc.indexIn(newVersion) > -1){
			rc= regRc.cap(1).toInt();
		}
		compare(major, minor, patch, rc);
		return;
	}
	
	QNetworkAccessManager *http = new QNetworkAccessManager(this);
	connect(http, SIGNAL(finished(QNetworkReply*)),
         this, SLOT(replyFinished(QNetworkReply*)));
	QNetworkRequest request(QUrl("http://ftp.engsas.de/versions"));
	request.setRawHeader("User-Agent", QString("Meganizer %1").arg(VERSION).toAscii());
	http->get(request);
}

void UpdateNoticeDialog::openLink(QString url)
{
	OpenFileExternal openFile(this);
	connectByPass(&openFile);
	openFile.openFile(url);
	accept();
}

void UpdateNoticeDialog::dontShowAgain(int state)
{
	if(state == Qt::PartiallyChecked)
		return;
	
	QSettings settings;
	bool checked = false;
	if(state == Qt::Checked)
		checked = true;
	// since the dialog appears after setting the current available version,
	// we can use it here to store it as the version, for which no updates
	// are displayed anymore
	if(checked)
		settings.setValue("dontShowUpdateNotice", settings.value("newAppVersion").toString());
}

void UpdateNoticeDialog::replyFinished(QNetworkReply *reply)
{
	if(platform.isEmpty())
		return;
		
	if(reply->error() != QNetworkReply::NoError)
		return;

	QSettings settings;
	QDomDocument xmlDoc;
	xmlDoc.setContent(reply->readAll());
	reply->deleteLater();
	QDomElement elem = xmlDoc.firstChildElement(settings.value("appNameShort").toString());
	if(elem.isNull()){
		emit information(tr("Could not get version information!"));
		return;
	}
	elem = elem.firstChildElement("platform");

	int major = 0;
	int minor = 0;
	int patch = 0;
	int rc = 0;
	while(!elem.isNull()){
		if(elem.attribute("name") == platform){
			elem = elem.firstChildElement("actualVersion");
			if(!elem.isNull()){
				major = elem.attribute("major", QString("0")).toInt();
				minor = elem.attribute("minor", QString("0")).toInt();
				patch = elem.attribute("patch", QString("0")).toInt();
				rc = elem.attribute("rc", QString("0")).toInt();
			}
		}
		elem = elem.nextSiblingElement("platform");
	}
	compare(major, minor, patch, rc);
}

void UpdateNoticeDialog::compare(int major, int minor, int patch, int rc)
{
	if((major == 0) && (minor == 0) && (patch == 0) && (rc == 0)){
		emit information(tr("Could not determine new version!"));
		return;
	}

	QSettings settings;
	QString versionString = QString("%1.%2.%3-rc%4").arg(major).arg(minor).arg(patch).arg(rc);
	if(versionString == settings.value("dontShowUpdateNotice"))
		return;
	// save actual version
	settings.setValue("newAppVersion", versionString);
	
	// get our version
	QString curVersion = settings.value("appVersion").toString();
	QRegExp regVer("([0-9]+)\\.([0-9]+)\\.([0-9]+)");
	if(regVer.indexIn(curVersion) == -1)
		return;
	int curMajor = regVer.cap(1).toInt();
	int curMinor = regVer.cap(2).toInt();
	int curPatch = regVer.cap(3).toInt();
	int curRc = 0;
	QString rcString;
	QRegExp regRc("-rc([0-9]+)");
	if(regRc.indexIn(curVersion) > -1){
		curRc = regRc.cap(1).toInt();
	}

	bool updatePossible = false;
	if(major > curMajor)
		updatePossible = true;
	if(minor > curMinor && !updatePossible)
		updatePossible = true;
	if(patch > curPatch && !updatePossible)
		updatePossible = true;
	// think of switching from 0.3.0-rc2 to 0.3.0
	// therefore a switch from rc < curRc if rc == 0 must be an update
	if(rc > curRc || (rc == 0 && major == curMajor && minor == curMinor && patch == curPatch))
		updatePossible = true;

	settings.setValue("updateCheck", QDate::currentDate());

	if(updatePossible){
		QString url = settings.value("appWWW").toString();
		QString rcString;
		if(rc > 0)
			rcString = QString("-rc%1").arg(rc);
		labelInformation->setText("<h3>"
															+ tr("An Update is available (Actual Version: %1.%2.%3%4)!")
																.arg(major).arg(minor).arg(patch).arg(rcString)
															+ "</h3>"
															+ tr("Please download new version at %1").arg(QString(" <a href=\"%1\">%1</a>.").arg(url))
														);
		exec();
	}
	else{
		emit information(tr("No update available!"));
	}
	close();
}

#include "updatenoticedialog.moc"
